//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//


#include <owl/pch.h>

#include "delgadaview.h"
#include "delgadadoc.h"
#include <owl/buttonga.h>
#include <owl/docmanag.h>
#include <owl/controlb.h>

#include "textgadgeteneblavel.hpp"
#include "dagoapp.h"
#include "dialogos.h"

REGISTRATION_FORMAT_BUFFER(1000)

DEFINE_DOC_TEMPLATE_CLASS(TDelgadaDoc, TDelgadaView, DocType2);

BEGIN_REGISTRATION(doctype2registro)
  REGDATA(description, "Sees delgadas")
  REGDATA(docfilter, "*.ssd")
  REGDATA(extension, "ssd")
  REGDOCFLAGS(dtAutoDelete | dtAutoOpen)
END_REGISTRATION
     
DocType2 __dvt2 (doctype2registro);

DEFINE_RESPONSE_TABLE1(TDagoApp, TApplication)
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_TUTORIAL, CmTutorial),
END_RESPONSE_TABLE;

TDagoApp::TDagoApp() : TApplication("Flexo II")
{
  SetDocManager(new TDocManager(dmSDI, this));
}

TDagoApp::~TDagoApp()
{
}

void TDagoApp::SetupControlBar(TDecoratedFrame* frame)
{
  TControlBar* cb = new TControlBar(frame);

  cb->Insert(*new TButtonGadget(CM_COMANDOUNDO,CM_COMANDOUNDO));
  cb->Insert(*new TButtonGadget(CM_COMANDOREDO,CM_COMANDOREDO));
  cb->Insert(*new TSeparatorGadget(12));
  cb->Insert(*new TButtonGadget(CM_COMANDOSECAO,CM_COMANDOSECAO));
  cb->Insert(*new TButtonGadget(CM_COMANDOTESOURA,CM_COMANDOTESOURA));
  cb->Insert(*new TButtonGadget(CM_COMANDODISTANCIAS,CM_COMANDODISTANCIAS));
  cb->Insert(*new TButtonGadget(CM_COMANDOBORRACHA,CM_COMANDOBORRACHA));
  cb->Insert(*new TButtonGadget(CM_COMANDOORIGEM,CM_COMANDOORIGEM));
  cb->Insert(*new TSeparatorGadget(12));
  cb->Insert(*new TButtonGadget(CM_MOSTRARD,CM_MOSTRARD));
  cb->Insert(*new TButtonGadget(CM_MOSTRARSETORIAL,CM_MOSTRARSETORIAL));
  cb->Insert(*new TButtonGadget(CM_MOSTRARCGEIXO,CM_MOSTRARCGEIXO));
  cb->Insert(*new TButtonGadget(CM_MOSTRARKCIRCULO,CM_MOSTRARKCIRCULO));
  cb->Insert(*new TButtonGadget(CM_MOSTRARESPESSURAS,CM_MOSTRARESPESSURAS));
  cb->Insert(*new TButtonGadget(CM_MOSTRARDISTORIG,CM_MOSTRARDISTORIG));
  cb->Insert(*new TSeparatorGadget(12));
  cb->Insert(*new TButtonGadget(CM_MOSTRARBAGUNCADO,CM_MOSTRARBAGUNCADO));
  cb->Insert(*new TButtonGadget(CM_COMANDOZOOM,CM_COMANDOZOOM));
  cb->Insert(*new TButtonGadget(CM_COMANDOMAOZINHA,CM_COMANDOMAOZINHA));
  cb->Insert(*new TButtonGadget(CM_COMANDOMETAFILE,CM_COMANDOMETAFILE));

  TControlBar* cb2 = new TControlBar(frame,TControlBar::Vertical);
  
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOAREA,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOIX,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOIY,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOCG,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOTETAX,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOJT,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOD,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOIW,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOK,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOR,8,TTextGadget::Recessed,TTextGadget::Left,12));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTOFECHADA,8,TTextGadget::Recessed,TTextGadget::Left,12,"Seo fechada"));
  cb2->Insert(*new TTextGadgetEneblavel(CM_TEXTODESCONTINUA,8,TTextGadget::Recessed,TTextGadget::Left,12,"Seo descontnua"));

  cb->SetHintMode(TGadgetWindow::EnterHints);
  cb2->SetHintMode(TGadgetWindow::EnterHints);
  frame->Insert(*cb, TDecoratedFrame::Top);
  frame->Insert(*cb2, TDecoratedFrame::Right);
}

void TDagoApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TDecoratedFrame* frame = new TDecoratedFrame(0, GetName(), 0, false, this);
  (new SplashDialog(frame))->EnableAutoCreate();

  frame->SetIcon(this, IDI_APPLICATION);
  frame->SetIconSm(this, IDI_APPLICATION);
  frame->AssignMenu(IDM_MENUPRINCIPAL);

  SetupControlBar(frame);

  SetMainWindow(frame);
}

void TDagoApp::InitInstance()
{
  TApplication::InitInstance();
  GetDocManager()->CreateAnyDoc(0, dtNewDoc);
}

void TDagoApp::EvNewView(TView& view)
{
  GetMainWindow()->SetClientWindow(view.GetWindow());
  if (!view.IsOK())
    GetMainWindow()->SetClientWindow(0);
}

void TDagoApp::EvCloseView(TView&)
{
  GetMainWindow()->SetClientWindow(0);
  GetMainWindow()->SetCaption(GetName());
}

void TDagoApp::CmHelpAbout()
{
  SplashDialog(GetMainWindow()).Execute();
}

int OwlMain(int , char* [])
{
  TDagoApp   app;
  return app.Run();
}

void TDagoApp::CmTutorial()
{
  STARTUPINFO si;
  PROCESS_INFORMATION pi;

  si.cb = sizeof si;
  si.lpReserved = NULL;
  si.lpDesktop = NULL;
  si.lpTitle = NULL;
  si.dwFlags = 0;
  si.cbReserved2 = 0;
  si.lpReserved2 = NULL;

  char buffer[2000];
  if (!GetModuleFileName(buffer,2000)) return;
  char* paux = buffer;
  while (*paux++);
  while (*--paux != '\\');
  strcpy(paux+1,"manual\\index.html");

  ::ShellExecute(0,"open",buffer,NULL,NULL,0);
}

