//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//


#include <owl/pch.h>

#include <algorithm>
#include <utility>
#include <vector>

#include "delgadadoc.h"


using namespace std;

TDelgadaDoc::TDelgadaDoc(TDocument* parent)
:
    TFileDocument(parent)
{
  CleanDoc ();
}

TDelgadaDoc::~TDelgadaDoc()
{
  CleanDoc ();
}

namespace {inline void deletar(TDo_Undo* p) {delete p;};}

void
TDelgadaDoc::CleanDoc ()
{
  for_each(do_undo_list.begin(),do_undo_list.end(),deletar);
  do_undo_list.resize(0);
  next = do_undo_list.begin();

  calculos_atualizados = false;

  JaFoiEscalado = false;
  
  cotas.resize(0);
  trechos.resize(0);
  distancias.resize(0);
  cxOrigem = cyOrigem = -1;
}

void
TDelgadaDoc::ConverterParaEixosPrincipais (template_point<double>& p) const
{
  template_point<double> paux (p.x - CG.x , p.y - CG.y);
  p.x = paux.x*cos(teta_xprincipal) + paux.y*sin(teta_xprincipal);
  p.y = paux.y*cos(teta_xprincipal) - paux.x*sin(teta_xprincipal);
}
void
TDelgadaDoc::ConverterParaEixosNormais (template_point<double>& p) const
{
  template_point<double> paux;
  paux.x = p.x*cos(teta_xprincipal) - p.y*sin(teta_xprincipal);
  paux.y = p.y*cos(teta_xprincipal) + p.x*sin(teta_xprincipal);
  p = paux + template_size<double>(CG.x,CG.y);
}
bool
TDelgadaDoc::PropTrecho(const trecho&t,template_point<double>&p1,template_point<double>&p2,
                        template_size<double>& sz,double& L,double& e,bool usar_tetax_CG) const
{
  if (!TrechoExiste(t)) return false;
  p1.x = cotas[t.cx1].valor;
  p1.y = cotas[t.cy1].valor;
  p2.x = cotas[t.cx2].valor;
  p2.y = cotas[t.cy2].valor;
  if (usar_tetax_CG){
    ConverterParaEixosPrincipais(p1);
    ConverterParaEixosPrincipais(p2);
  };
  sz = p2 - p1;
  L = sqrt(sz.cx*sz.cx + sz.cy*sz.cy);
  e = t.espessura;

  return true;
}
bool
TDelgadaDoc::TrechoMalPreenchido(const trecho& t) const
{
  if (t.cx1 < 0 || t.cx2 < 0 || t.cy1 < 0 || t.cy2 < 0) return true;
  int maxc = cotas.size() - 1;
  if (t.cx1 > maxc || t.cx2 > maxc || t.cy1 > maxc || t.cy2 > maxc) return true;
  if (!( cotas[t.cx1].e==eixo_x && cotas[t.cx2].e==eixo_x
      && cotas[t.cy1].e==eixo_y && cotas[t.cy2].e==eixo_y)) return true;
  return false;
}
bool
TDelgadaDoc::DistanciaMalPreenchida (const DistanciaEntreCotas& d) const
{
  if (d.c1 < 0 || d.c2 < 0 || d.c1 >= cotas.size() || d.c2 >= cotas.size())
    return true;
  if (cotas[d.c1].e != cotas[d.c2].e) return true;
  return false;
}
bool
TDelgadaDoc::TrechoExiste(const trecho& t) const
{
  CHECK(!TrechoMalPreenchido(t));
  if (!t.valid) return false;
  if (!CotaExiste(cotas[t.cx1])) return false;
  if (!CotaExiste(cotas[t.cx2])) return false;
  if (!CotaExiste(cotas[t.cy1])) return false;
  if (!CotaExiste(cotas[t.cy2])) return false;
  return true;
}
bool
TDelgadaDoc::DistanciaExiste (const DistanciaEntreCotas& d) const
{
  CHECK (!DistanciaMalPreenchida(d));
  if (!d.valid) return false;
  if (!CotaExiste(cotas[d.c1]) || !CotaExiste(cotas[d.c2])) return false;
  return true;
}
bool
TDelgadaDoc::CotaExiste (const cota& c) const
{
  if (!c.valid) return false;
  return true;
}


void
TDelgadaDoc::AddDoUndo (TDo_Undo* p)
{
  list<TDo_Undo*>::iterator new_next = do_undo_list.insert(next,p);

  for_each(next,do_undo_list.end(),deletar);
  do_undo_list.erase(next,do_undo_list.end());

  next = new_next;
}
bool
TDelgadaDoc::CanDo() const
{
  if (do_undo_list.end() == next) return false;
  return true;
}
bool TDelgadaDoc::CanUndo() const
{
  if (do_undo_list.begin() == next) return false;
  return true;
}
void TDelgadaDoc::Do()
{
  if (!CanDo()) return;
  (*(next++))->Do();
  SetDirty(true);
}
void TDelgadaDoc::Undo()
{
  if (!CanUndo()) return;
  (*(--next))->Undo();
  SetDirty(true);
}

void TDelgadaDoc::AtualizarCalculos()
{
  if (calculos_atualizados) return;

  secao_fechada = secao_descontinua = false;
  Area_CG_Jt_MomentosPrincipais_EixoPrincipal();
  AreaSetorial_D_Iw();
  Kx_Ky_R();

  calculos_atualizados = true;
}
void TDelgadaDoc::Area_CG_Jt_MomentosPrincipais_EixoPrincipal ()
{
  Area = Jt = 0;

  double Msx,Msy,Ix0,Iy0,Ixy0;
  Msx = Msy = Ix0 = Iy0 = Ixy0 = 0;

  template_point<double> p1,p2;
  template_size<double> sz;
  double L,espessura;

  for (vector<trecho>::const_iterator i=trechos.begin();i!=trechos.end();i++)
    if (PropTrecho((*i),p1,p2,sz,L,espessura,false)){
      double Atrecho = L * espessura;
      double Ixtrecho = pow(L,3)*espessura/12;
      double Iytrecho = L*pow(espessura,3)/12;
      double xmedio = (p1.x + p2.x) / 2;
      double ymedio = (p1.y + p2.y) / 2;
      double c = sz.cy/L;
      double s = sz.cx/L;


      Area += Atrecho;
      Jt += espessura*espessura*Atrecho;

      Msx += Atrecho * ymedio;
      Msy += Atrecho * xmedio;

      Ix0  += c*c*Ixtrecho + s*s*Iytrecho;
      Iy0  += s*s*Ixtrecho + c*c*Iytrecho;
      Ixy0 += s*c*(Ixtrecho - Iytrecho);
      Ix0  += Atrecho*ymedio*ymedio;
      Iy0  += Atrecho*xmedio*xmedio;
      Ixy0 += Atrecho*xmedio*ymedio;

    };

  if (Area!=0) {
    Jt /= 3;

    CG.x = Msy/Area;
    CG.y = Msx/Area;

    Ix0  -= Area * CG.y * CG.y;
    Iy0  -= Area * CG.x * CG.x;
    Ixy0 -= Area * CG.x * CG.y;

    teta_xprincipal = atan(-2*Ixy0/(Ix0-Iy0))/2;
    double s = sin(teta_xprincipal);
    double c = cos(teta_xprincipal);
    Iprincipal.x = Ix0*c*c - 2*Ixy0*s*c + Iy0*s*s;
    Iprincipal.y = Ix0*s*s + 2*Ixy0*s*c + Iy0*c*c;
  };
}
void TDelgadaDoc::Kx_Ky_R ()
{
  if (Area==0) return;

  template_point<double> p1,p2;
  template_size<double> sz;
  double L,espessura;
  double IntegralKx,IntegralKy; IntegralKx = IntegralKy = 0;
  for (vector<trecho>::const_iterator i=trechos.begin();i!=trechos.end();i++)
    if (PropTrecho((*i),p1,p2,sz,L,espessura,true)){
      double x3,x2y,xy2,y3; // Pedaos das integrais

      x3 = (pow(p1.x,3) + p1.x*p2.x*p2.x + p1.x*p1.x*p2.x + pow(p2.x,3)) * L / 4;
      y3 = (pow(p1.y,3) + p1.y*p2.y*p2.y + p1.y*p1.y*p2.y + pow(p2.y,3)) * L / 4;
      x2y = (2*p1.x*p2.x*p1.y + 2*p1.x*p2.x*p2.y + 3*p1.x*p1.x*p1.y +
             p2.x*p2.x*p1.y + 3*p2.x*p2.x*p2.y + p1.x*p1.x*p2.y) * L / 12;
      xy2 = (2*p1.y*p2.y*p1.x + 2*p1.y*p2.y*p2.x + 3*p1.y*p1.y*p1.x +
             p2.y*p2.y*p1.x + 3*p2.y*p2.y*p2.x + p1.y*p1.y*p2.x) * L / 12;

      IntegralKx += x3 + xy2;
      IntegralKy += y3 + x2y;
    };

  K.x = IntegralKx / (2 * Iprincipal.y);
  K.y = IntegralKy / (2 * Iprincipal.x);

  R = sqrt(K.x*K.x + K.y*K.y + (Iprincipal.y + Iprincipal.x)/Area);

  ConverterParaEixosNormais(K);
}
void TDelgadaDoc::AreaSetorial_D_Iw ()
{
  if (Area==0) return;

  vector<vector<double> > matriz;
  vector<vector<bool> > mbool;

  D = CG; //template_point<bcd>(0,0);

  AreaSetorialDadoD(matriz,mbool);

  double Integral_wp_x, Integral_wp_y; Integral_wp_x = Integral_wp_y = 0;

  for (vector<trecho>::const_iterator i=trechos.begin();i!=trechos.end();i++){
    template_point<double> p1,p2;
    template_size<double> sz;
    double L,espessura;
    if (PropTrecho ((*i),p1,p2,sz,L,espessura,true)){
      double w1,w2;
      w1 = matriz[(*i).cx1][(*i).cy1];
      w2 = matriz[(*i).cx2][(*i).cy2];

      Integral_wp_x += ( p1.x*w1/3 + p1.x*w2/6 + p2.x*w1/6 + p2.x*w2/3 )*L*espessura;
      Integral_wp_y += ( p1.y*w1/3 + p1.y*w2/6 + p2.y*w1/6 + p2.y*w2/3 )*L*espessura;
    };
  };


//  ConverterParaEixosPrincipais(D);
  D.x = -Integral_wp_y/Iprincipal.x;
  D.y = Integral_wp_x/Iprincipal.y;
  ConverterParaEixosNormais(D);

  AreaSetorialDadoD(matriz,mbool);

  double Integral_wp_S = 0;

  for (vector<trecho>::const_iterator i=trechos.begin();i!=trechos.end();i++){
    template_point<double> p1,p2;
    template_size<double> sz;
    double L,espessura;
    if (PropTrecho ((*i),p1,p2,sz,L,espessura,false)){
      double w1,w2;
      w1 = matriz[(*i).cx1][(*i).cy1];
      w2 = matriz[(*i).cx2][(*i).cy2];

      Integral_wp_S += ( w1 + w2 ) / 2 * L * espessura;
    };
  };

  double C = Integral_wp_S / Area;
  for (int i = 0; i < cotas.size(); i++)
    for (int j = 0; j < cotas.size(); j++)
      matriz[i][j] -= C;

  Iw = 0;

  for (vector<trecho>::const_iterator i=trechos.begin();i!=trechos.end();i++){
    template_point<double> p1,p2;
    template_size<double> sz;
    double L,espessura;
    if (PropTrecho ((*i),p1,p2,sz,L,espessura,false)){
      double w1,w2;
      w1 = matriz[(*i).cx1][(*i).cy1];
      w2 = matriz[(*i).cx2][(*i).cy2];

      Iw += ( w1*w1 + w1*w2 + w2*w2 ) / 3 * L * espessura;
    };
  };

  ws_dos_vertices.resize(0);
  posicao_dos_vertices.resize(0);

  for (int i = 0; i < cotas.size(); i++)
    for (int j = 0; j < cotas.size(); j++)
      if (mbool[i][j]){
        ws_dos_vertices.push_back(matriz[i][j]);
        posicao_dos_vertices.push_back(template_point<double>(cotas[i].valor,cotas[j].valor));
      };
}
void TDelgadaDoc::AreaSetorialDadoD (vector<vector<double> >& matriz,vector<vector<bool> >& mbool)
{
  matriz.assign(cotas.size(),vector<double>(cotas.size(),0));
  mbool.assign(cotas.size(),vector<bool>(cotas.size(),false));
  vector<trecho> copy_trechos = trechos;
  vector<trecho>::iterator i1,i2,end;
  i2 = copy_trechos.end();
  bool tudo_vazio = true;
  do {
    end = i2;
    for (i1 = i2 = copy_trechos.begin(); i1 != end; i1++){
      template_point<double>p1,p2;template_size<double>sz;double L,e;
      if (PropTrecho(*i1,p1,p2,sz,L,e,false)){
        int cx1 = (*i1).cx1; int cy1 = (*i1).cy1;
        int cx2 = (*i1).cx2; int cy2 = (*i1).cy2;
        if (tudo_vazio) {tudo_vazio = false; mbool[cx1][cy1] = true;};
        bool bool1 = mbool[cx1][cy1];
        bool bool2 = mbool[cx2][cy2];
        if (bool1 && bool2) secao_fechada = true; // Seo fechada.
        if (bool1 || bool2) {
          if (bool2) {swap (cx1,cx2); swap (cy1,cy2); swap (p1,p2);};
          p1 -= D - template_point<double>(0,0);
          p2 -= D - template_point<double>(0,0);
          double acrescimo = p2.x*p1.y - p1.x*p2.y;
          matriz[cx2][cy2] = matriz[cx1][cy1] + acrescimo;
          mbool[cx2][cy2] = true;
        }
        else swap (*i1,*i2++); // Guardar para o prximo lao.
      };
    };
  } while (end != i2);

  secao_descontinua = i2 != copy_trechos.begin(); // Caso !=, alguns trechos esto soltos.
}

bool TDelgadaDoc::LimitesDaSecao (double& min,double& max,eixo e) const
{
  bool tem = false;
  for (vector<cota>::const_iterator i = cotas.begin();i!=cotas.end();i++)
    if (CotaExiste(*i))
      if ((*i).e == e){
        double val = (*i).valor;
        if (!tem) min = max = val;
        if (val < min) min = val;
        if (val > max) max = val;
        tem = true;
      };
  for (vector<DistanciaEntreCotas>::const_iterator i = distancias.begin();i!=distancias.end();i++)
    if (DistanciaExiste(*i))
      if (cotas[(*i).c1].e != e){
        double val = (*i).posicao_ortogonal;
        if (!tem) min = max = val;
        if (val < min) min = val;
        if (val > max) max = val;
        tem = true;
      };
  return tem;
}

bool TDelgadaDoc::GetArea(double& A)
{
  if (!calculos_atualizados) AtualizarCalculos();
  A = Area;
  return true;
}
bool TDelgadaDoc::GetCG (template_point<double>& CG_)
{
  if (!calculos_atualizados) AtualizarCalculos();
  CG_ = CG;
  return Area > 0;
}
bool TDelgadaDoc::GetMomentosPrincipais (template_point<double>& I)
{
  if (!calculos_atualizados) AtualizarCalculos();
  I = Iprincipal;
  return Area > 0;
}
bool TDelgadaDoc::GetTeta_xprincipal (double& teta_xprincipal_)
{
  if (!calculos_atualizados) AtualizarCalculos();
  teta_xprincipal_ = teta_xprincipal;
  return Area > 0;
}
bool TDelgadaDoc::GetJt (double& Jt_)
{
  Jt_ = Jt;
  return Area > 0;
}
bool TDelgadaDoc::GetK (template_point<double>& K_)
{
  if (!calculos_atualizados) AtualizarCalculos();
  K_ = K;
  return Area > 0;
}
bool TDelgadaDoc::GetR (double& R_)
{
  if (!calculos_atualizados) AtualizarCalculos();
  R_ = R;
  return Area > 0;
}
bool TDelgadaDoc::NaoHaDados()
{
  for (int i=0;i<cotas.size();i++)
    if (CotaExiste(cotas[i])) return false;
  return true; // Nenhum dado pode estar definido se no h cotas.
}
bool TDelgadaDoc::NaoHaDistancias()
{
  for (int i=0;i<distancias.size();i++)
    if (DistanciaExiste(distancias[i])) return false;
  return true;
}
bool TDelgadaDoc::NaoHaTrechos()
{
  for (int i=0;i<trechos.size();i++)
    if (TrechoExiste(trechos[i])) return false;
  return true;
}
bool TDelgadaDoc::SecaoFechada()
{
  if (!calculos_atualizados) AtualizarCalculos();
  return secao_fechada;
}
bool TDelgadaDoc::SecaoDescontinua()
{
  if (!calculos_atualizados) AtualizarCalculos();
  return secao_descontinua;
}
bool TDelgadaDoc::GetWs (vector<double>& v)
{
  if (!calculos_atualizados) AtualizarCalculos();
  v = ws_dos_vertices;
  return Area>0 && !secao_fechada && !secao_descontinua;
}
bool TDelgadaDoc::GetPosicao_dos_ws (vector<template_point<double> >& v)
{
  if (!calculos_atualizados) AtualizarCalculos();
  v = posicao_dos_vertices;
  return Area>0 && !secao_fechada && !secao_descontinua;
}
bool TDelgadaDoc::GetD (template_point<double>& D_)
{
  if (!calculos_atualizados) AtualizarCalculos();
  D_ = D;
  return Area>0 && !secao_fechada && !secao_descontinua;
}
bool TDelgadaDoc::GetIw (double& Iw_)
{
  if (!calculos_atualizados) AtualizarCalculos();
  Iw_ = Iw;
  return Area>0 && !secao_fechada && !secao_descontinua;
}
bool TDelgadaDoc::GetOrigem(template_point<double>&p)
{
  if (cxOrigem<0 || cyOrigem<0 || cxOrigem>=cotas.size() || cyOrigem>=cotas.size())
    return false;
  if (!CotaExiste(cotas[cxOrigem]) || !CotaExiste(cotas[cyOrigem]))
    return false;
  p.x = cotas[cxOrigem].valor;
  p.y = cotas[cyOrigem].valor;
  return true;
}


bool TDelgadaDoc::CotaNearestPonto (const double& p,int& cx, double& dist, eixo e) const
{
  bool temcota = false;
  for (vector<cota>::const_iterator i=cotas.begin(); i<cotas.end(); i++)
    if (CotaExiste(*i) && (*i).e==e){
      double distaux = fabs(p - (*i).valor);
      if (!temcota || distaux < dist){
        dist = distaux;
        cx = i - cotas.begin();
        temcota = true;
      }
    }
  return temcota;
}
namespace {
  bool NoMeio (double b1,double b2,const double& b3)
  {
    if (b1 > b2) swap (b1,b2);
    return b1 < b3 && b3 < b2;
  }
}
bool TDelgadaDoc::LinhaDeDistanciaNearestPonto (const template_point<double>& p, int& cd,double& dist) const
{
  bool temlinha = false;
  for (vector<DistanciaEntreCotas>::const_iterator i=distancias.begin(); i<distancias.end(); i++)
    if (DistanciaExiste(*i))
      if (NoMeio(cotas[(*i).c1].valor,cotas[(*i).c2].valor,(cotas[(*i).c1].e==eixo_x?p.x:p.y))){
        double distaux = fabs((*i).posicao_ortogonal - (cotas[(*i).c1].e==eixo_x?p.y:p.x));
        if (!temlinha || distaux < dist) {
          dist = distaux;
          cd = i - distancias.begin();
          temlinha = true;
        };
      };
  return temlinha;
}
bool TDelgadaDoc::TrechoDeSecaoNearestPonto (const template_point<double>& p, int& it ,double& dist) const
{
  bool temtrecho = false;
  for (vector<trecho>::const_iterator i=trechos.begin(); i<trechos.end(); i++)
    if (TrechoExiste(*i)){
      double x1,y1,x2,y2;
      double dx,dy,c1,c2,c3,d1,d3;

      x1 = cotas[(*i).cx1].valor; y1 = cotas[(*i).cy1].valor;
      x2 = cotas[(*i).cx2].valor; y2 = cotas[(*i).cy2].valor;
      dx = x1 - x2;
      dy = y1 - y2;
      c1 = dx*x1 + dy*y1;
      c2 = dx*x2 + dy*y2;
      c3 = dx*p.x + dy*p.y;
      if (c1>c2) swap (c1,c2);

      if (c3>c1 && c3<c2){ //Note-se o uso de < e no de <=. Isso  importante para
        d1 = dy*x1 - dx*y1;  //evitar que um toque na extremidade de um trecho aponte
        d3 = dy*p.x - dx*p.y;  //para as cotas na extremidade, e no para o trecho.
        double distaux = fabs((d3-d1)/sqrt(dx*dx+dy*dy));
        if (!temtrecho || distaux < dist){
          dist = distaux;
          it = i - trechos.begin();
          temtrecho = true;
        }
      }
    }

  return temtrecho;
}
bool TDelgadaDoc::CotasQueMudariamSeMudasseDistancia (int idist, vector<bool>& v) const
{
  if (idist >= distancias.size()) return false;
  if (!DistanciaExiste(distancias[idist])) return false;

  v.assign(cotas.size(),false);
  vector<DistanciaEntreCotas> copy_distancias = distancias;

  int c1 = distancias[idist].c1;
  int c2 = distancias[idist].c2;

  v[cotas[c1].valor>cotas[c2].valor?c1:c2] = true;

  vector<DistanciaEntreCotas>::iterator i1,i2,end;

  end = copy_distancias.end();

  for (i1 = i2 = copy_distancias.begin(); i1 != end; i1++){
    if (!DistanciaExiste(*i1)) continue;
    if ((*i1).c1 == c1 && (*i1).c2 == c2) continue;
    if ((*i1).c1 == c2 && (*i1).c2 == c1) continue;
    swap (*i1,*i2++);
  }
  do {
    end = i2;
    for (i1 = i2 = copy_distancias.begin(); i1 != end; i1++)
      if (v[(*i1).c1] || v[(*i1).c2]) v[(*i1).c1] = v[(*i1).c2] = true;
      else swap (*i1,*i2++);
  } while (end!=i2);

  return !v[cotas[c1].valor>cotas[c2].valor?c2:c1];
}
bool TDelgadaDoc::DadosParaDesenharSecao (vector<pair<template_point<double>,template_point<double> > >& v,vector<double>& e) const
{
  v.resize(0);
  e.resize(0);
  for (vector<trecho>::const_iterator i=trechos.begin();i!=trechos.end();i++){
    template_point<double>p1,p2;template_size<double>sz;double L,espessura;
    if (PropTrecho(*i,p1,p2,sz,L,espessura,false)){
      v.push_back(make_pair(p1,p2));
      e.push_back(espessura);
    };
  };
  return true;
}
bool TDelgadaDoc::Close()
{
  if (!TFileDocument::Close()) return false;

  CleanDoc ();

  return true;
}


bool TDelgadaDoc::Commit(bool force)
{
  if (!IsDirty() && !force) return true;

  TOutStream* os = OutStream(ofWrite);
  if (!os) return false;

  *os << JaFoiEscalado << ' ';
  *os << cotas.size() << ' ';
  *os << trechos.size() << ' ';
  *os << distancias.size() << ' ';

  for (vector<cota>::iterator i=cotas.begin();i!=cotas.end();i++)
    *os << (*i).valid << ' ' << (*i).valor << ' ' << ((*i).e==eixo_x?1:0) << ' ';
  for (vector<trecho>::iterator i=trechos.begin();i!=trechos.end();i++)
    *os << (*i).valid << ' ' << (*i).cx1 << ' ' << (*i).cx2 << ' ' << (*i).cy1
        << ' ' << (*i).cy2 << ' ' << (*i).espessura << ' ';
  for (vector<DistanciaEntreCotas>::iterator i=distancias.begin();i!=distancias.end();i++)
    *os << (*i).valid << ' ' << (*i).c1 << ' ' << (*i).c2 << ' ' << (*i).posicao_ortogonal << ' ';

  *os << cxOrigem << ' ' << cyOrigem << ' '; //O espao extra  s para ajudar na checagem da leitura. Vide Open()

  delete os;

  SetDirty(false);
  return true;
}


bool TDelgadaDoc::Open(int mode, const char far* path)
{
  if (path)
    SetDocPath(path);

  CleanDoc();

  if (GetDocPath()) {
    TInStream* is = InStream(ofRead);
    if (!is) return false;


    int csize,tsize,dsize;

    *is >> JaFoiEscalado;
    *is >> csize >> tsize >> dsize;

    while (csize--){
      cota c;
      int aux;
      *is >> c.valid >> c.valor >> aux;
      c.e = aux==1?eixo_x:eixo_y;
      cotas.push_back(c);
    };
    while (tsize--){
      trecho t;
      *is >> t.valid >> t.cx1 >> t.cx2 >> t.cy1 >> t.cy2 >> t.espessura;
      trechos.push_back(t);
    };
    while (dsize--){
      DistanciaEntreCotas d;
      *is >> d.valid >> d.c1 >> d.c2 >> d.posicao_ortogonal;
      distancias.push_back(d);
    };

    *is >> cxOrigem >> cyOrigem;
    if ((*is).eof()) return false;

    delete is;
  };

  SetDirty(false);
  NotifyViews(vnRevert);
  return true;
}


bool TDelgadaDoc::Revert(bool clear)
{
  if (clear){
    CleanDoc ();
    NotifyViews(vnRevert);
  } else
    Open(0);

  return true;
}

