//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//


#if !defined(delgadadoc_h)
#define delgadadoc_h

#include <owl/filedoc.h>

#include <list>
#include <map>
#include <set>
#include <functional>
#include <utility>
#include <vector>

#include "dagoapp.rh"

#include <do_undo.hpp>
#include <pointsizerect.hpp>

enum eixo {eixo_x,eixo_y};

struct cota {
  bool valid;
  double valor;
  eixo e; // true  x, false  y
};
struct trecho {
  bool valid;
  int cx1,cx2,cy1,cy2; //ndices de cotas
  double espessura;
};
struct DistanciaEntreCotas {
  bool valid;
  int c1;
  int c2;
  double posicao_ortogonal;
};

const int vnVouEscalar = vnCustomBase + 1;
NOTIFY_SIG(vnVouEscalar,double*)
#define EV_VN_VOUESCALAR VN_DEFINE(vnVouEscalar,VnVouEscalar,pointer)

class TDelgadaDoc : public TFileDocument {
  public:
    TDelgadaDoc(TDocument* parent = 0);
    virtual ~TDelgadaDoc();

  protected:
    void CleanDoc ();
    std::list<TDo_Undo*> do_undo_list;
    std::list<TDo_Undo*>::iterator next;

    bool calculos_atualizados;
    double Area;
    template_point<double> CG;
    template_point<double> Iprincipal;
    double teta_xprincipal;
    template_point<double> K;
    double R;
    bool secao_fechada;
    bool secao_descontinua;
    std::vector<double> ws_dos_vertices;
    std::vector<template_point<double> > posicao_dos_vertices;
    template_point<double> D;
    double Jt;
    double Iw;

    void ConverterParaEixosPrincipais (template_point<double>&) const;
    void ConverterParaEixosNormais (template_point<double>&) const;
    bool PropTrecho (const trecho&,template_point<double>&p1,template_point<double>&p2,
                     template_size<double>& sz,double& L,double& e,bool usar_tetax_CG) const;


  public:
    bool JaFoiEscalado;
    std::vector<cota> cotas;
    std::vector<trecho> trechos;
    std::vector<DistanciaEntreCotas> distancias;
    int cxOrigem,cyOrigem;

    bool TrechoMalPreenchido(const trecho&) const;
    bool DistanciaMalPreenchida (const DistanciaEntreCotas&) const;

    bool TrechoExiste(const trecho&) const;
    bool DistanciaExiste (const DistanciaEntreCotas&) const;
    bool CotaExiste (const cota&) const;


  public:
    void AddDoUndo (TDo_Undo*); //  Ser responsabilidade do DelgadaDoc deletar o objeto.
    bool CanDo() const;
    bool CanUndo() const;       //  Estas 5 funes membro so as nicas que as views devero
    void Do();                  //chamar para modificar o Doc.
    void Undo();

    void DesatualizarCalculos () {calculos_atualizados = false;};
    void AtualizarCalculos();
    void Area_CG_Jt_MomentosPrincipais_EixoPrincipal ();
    void Kx_Ky_R ();
    void AreaSetorial_D_Iw ();
    void AreaSetorialDadoD (std::vector<std::vector<double> >&,std::vector<std::vector<bool> >&);

    bool LimitesDaSecao (double&,double&,eixo) const;

    bool GetArea(double&);
    bool GetCG (template_point<double>&);
    bool GetMomentosPrincipais (template_point<double>&);
    bool GetTeta_xprincipal (double&);
    bool GetJt (double&);
    bool GetK (template_point<double>&);
    bool GetR (double&);
    bool NaoHaDados();
    bool NaoHaDistancias();
    bool NaoHaTrechos();
    bool SecaoFechada();
    bool SecaoDescontinua();
    bool GetWs (std::vector<double>&);
    bool GetPosicao_dos_ws (std::vector<template_point<double> >&);
    bool GetD (template_point<double>&);
    bool GetIw (double&);
    bool GetOrigem(template_point<double>&);

    bool CotaNearestPonto (const double& p,int& cx, double& dist, eixo e) const;
    bool LinhaDeDistanciaNearestPonto (const template_point<double>& p,int& cd,double& dist) const;
    bool TrechoDeSecaoNearestPonto (const template_point<double>& p, int& it ,double& dist) const;
    bool CotasQueMudariamSeMudasseDistancia (int idist, std::vector<bool>& v) const;
    bool DadosParaDesenharSecao (std::vector<std::pair<template_point<double>,template_point<double> > >&,std::vector<double>&) const;

  public:
    virtual bool Close();
    virtual bool Commit(bool force=false);
    virtual bool Open(int mode, const char far* path=0);
    virtual bool Revert(bool clear=false);
};


#endif  // delgadadoc_h

