//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//


#include <owl/pch.h>

#include <owl/dc.h>

#include "delgadaview.h"
#include "dialogos.h"

#include <do_undo_inheriteds.hpp>
#include <time.h>

using namespace std;

const int ZOOM = 1;
const int DISTANCIAS = 2;
const int SECAO = 4;
const int BORRACHA = 8;
const int MAOZINHA = 16;
const int ORIGEM = 32;
const int METAFILE = 64;
const int EDITANDO_DADOS = DISTANCIAS|SECAO|BORRACHA|ORIGEM;

const int MOSTRAR_ESPESSURA = 1;
const int MOSTRAR_CGEIXOS = 2;
const int MOSTRAR_KCIRCULO = 4;
const int MOSTRAR_SETORIAL = 8;
const int MOSTRAR_DISTANCIASORIGEM = 16;
const int MOSTRAR_D = 32;
const int MOSTRAR_BAGUNCADO = 64;

const int distancia_maxima_entre_ponto_e_clique_do_mouse = 5;

DEFINE_RESPONSE_TABLE1(TDelgadaView, TWindowView)
  EV_WM_CAPTURECHANGED,
  EV_VN_REVERT,
  EV_VN_VOUESCALAR,
  EV_WM_LBUTTONDOWN,
  EV_WM_MOUSEMOVE,
  EV_WM_RBUTTONDOWN,
  EV_WM_LBUTTONUP,

  EV_COMMAND_ENABLE(CM_TEXTOAREA, CeTextoarea),
  EV_COMMAND_ENABLE(CM_TEXTOCG, CeTextocg),
  EV_COMMAND_ENABLE(CM_TEXTOD, CeTextod),
  EV_COMMAND_ENABLE(CM_TEXTOIW, CeTextoiw),
  EV_COMMAND_ENABLE(CM_TEXTOIX, CeTextoix),
  EV_COMMAND_ENABLE(CM_TEXTOIY, CeTextoiy),
  EV_COMMAND_ENABLE(CM_TEXTOJT, CeTextojt),
  EV_COMMAND_ENABLE(CM_TEXTOK, CeTextok),
  EV_COMMAND_ENABLE(CM_TEXTOR, CeTextor),
  EV_COMMAND_ENABLE(CM_TEXTOTETAX, CeTextotetax),
  EV_COMMAND_ENABLE(CM_TEXTOFECHADA, CeTextofechada),
  EV_COMMAND_ENABLE(CM_TEXTODESCONTINUA, CeTextodescontinua),

  EV_COMMAND_ENABLE(CM_MOSTRARCGEIXO, CeMostrarcgeixo),
  EV_COMMAND_ENABLE(CM_MOSTRARD, CeMostrard),
  EV_COMMAND_ENABLE(CM_MOSTRARDISTORIG, CeMostrardistorig),
  EV_COMMAND_ENABLE(CM_MOSTRARESPESSURAS, CeMostrarespessuras),
  EV_COMMAND_ENABLE(CM_MOSTRARKCIRCULO, CeMostrarkcirculo),
  EV_COMMAND_ENABLE(CM_MOSTRARSETORIAL, CeMostrarsetorial),
  EV_COMMAND_ENABLE(CM_MOSTRARBAGUNCADO, CeMostrarbaguncado),

  EV_COMMAND_ENABLE(CM_COMANDOREDO, CeComandoredo),
  EV_COMMAND(CM_COMANDOREDO, CmComandoredo),
  EV_COMMAND_ENABLE(CM_COMANDOUNDO, CeComandoundo),
  EV_COMMAND(CM_COMANDOUNDO, CmComandoundo),
  EV_COMMAND_ENABLE(CM_COMANDOBORRACHA, CeComandoborracha),
  EV_COMMAND(CM_COMANDOBORRACHA, CmComandoborracha),
  EV_COMMAND_ENABLE(CM_COMANDODISTANCIAS, CeComandodistancias),
  EV_COMMAND(CM_COMANDODISTANCIAS, CmComandodistancias),
  EV_COMMAND_ENABLE(CM_COMANDOMAOZINHA, CeComandomaozinha),
  EV_COMMAND(CM_COMANDOMAOZINHA, CmComandomaozinha),
  EV_COMMAND_ENABLE(CM_COMANDOSECAO, CeComandosecao),
  EV_COMMAND(CM_COMANDOSECAO, CmComandosecao),
  EV_COMMAND_ENABLE(CM_COMANDOTESOURA, CeComandotesoura),
  EV_COMMAND(CM_COMANDOTESOURA, CmComandotesoura),
  EV_COMMAND_ENABLE(CM_COMANDOZOOM, CeComandozoom),
  EV_COMMAND(CM_COMANDOZOOM, CmComandozoom),
  EV_COMMAND_ENABLE(CM_COMANDOORIGEM, CeComandoorigem),
  EV_COMMAND(CM_COMANDOORIGEM, CmComandoorigem),
  EV_COMMAND_ENABLE(CM_COMANDOMETAFILE, CeComandometafile),
  EV_COMMAND(CM_COMANDOMETAFILE, CmComandometafile),

  EV_WM_SIZE,

  EV_COMMAND_AND_ID(CM_MOSTRARCGEIXO,CmComandosmostrar),
  EV_COMMAND_AND_ID(CM_MOSTRARD,CmComandosmostrar),
  EV_COMMAND_AND_ID(CM_MOSTRARDISTORIG,CmComandosmostrar),
  EV_COMMAND_AND_ID(CM_MOSTRARESPESSURAS,CmComandosmostrar),
  EV_COMMAND_AND_ID(CM_MOSTRARKCIRCULO,CmComandosmostrar),
  EV_COMMAND_AND_ID(CM_MOSTRARSETORIAL,CmComandosmostrar),
  EV_COMMAND_AND_ID(CM_MOSTRARBAGUNCADO,CmComandosmostrar),
END_RESPONSE_TABLE;


TDelgadaView::TDelgadaView(TDocument& doc, TWindow* parent)
:
    delgada_doc(dynamic_cast<TDelgadaDoc&>(doc)),
    TWindowView(doc, parent)
{
  x_no_zero = y_no_zero = 0;
  dreal_djanela = 1;
  NotSizedYet = true;
  que_fazer = que_mostrar = 0;
}


TDelgadaView::~TDelgadaView()
{
}

int
TDelgadaView::EscalarDistancia_dtoi (double d)
{
  return fabs(d/dreal_djanela);
}
double
TDelgadaView::EscalarDistancia_itod (int d)
{
  return fabs(d*dreal_djanela);
}
int
TDelgadaView::EscalarX_dtoi (double x)
{
  return (x-x_no_zero)/dreal_djanela;
}
double
TDelgadaView::EscalarX_itod (int x)
{
  return x_no_zero + x*dreal_djanela;
}
int
TDelgadaView::EscalarY_dtoi (double y)
{
  return -(y-y_no_zero)/dreal_djanela;
}
double
TDelgadaView::EscalarY_itod (int y)
{
  return y_no_zero - y*dreal_djanela;
}
template_point<double>
TDelgadaView::Escalar(TPoint p)
{
  return template_point<double>(EscalarX_itod(p.X()),EscalarY_itod(p.Y()));
}
TPoint
TDelgadaView::Escalar(template_point<double> p)
{
  return TPoint(EscalarX_dtoi(p.X()),EscalarY_dtoi(p.Y()));
}
template_rect<double> 
TDelgadaView::Escalar(TRect r)
{
  template_rect<double> r2 (Escalar(r.TopLeft()),Escalar(r.BottomRight()));
  return r2.Normalized();
}
TRect
TDelgadaView::Escalar(template_rect<double> r)
{
  TRect r2 (Escalar(r.minXminY()),Escalar(r.maxXmaxY()));
  return r2.Normalized();
}

void
TDelgadaView::Fit (TRect r,template_rect<double> rd)
{
  r.Normalize();
  rd.Normalize();
  PRECONDITION (!r.IsEmpty());

  int meioX = (r.Left() + r.Right())/2;
  int meioY = (r.Bottom() + r.Top())/2;
  dreal_djanela = rd.IsEmpty()?1:std::max(rd.Width()/r.Width(),rd.Height()/r.Height());
  x_no_zero = (rd.maxX()+rd.minX())/2 - meioX*dreal_djanela;
  y_no_zero = (rd.maxY()+rd.minY())/2 + meioY*dreal_djanela;
}

void
TDelgadaView::DesenharTodasAsCotas (TDC& dc)
{
  dc.SelectObject(TPen(TColor::LtGray,1,PS_DASHDOTDOT));
  vector<cota>::const_iterator i;
  for(i=delgada_doc.cotas.begin();i!=delgada_doc.cotas.end();i++)
    if (delgada_doc.CotaExiste(*i)){
      int cota = (*i).e==eixo_x?EscalarX_dtoi((*i).valor):EscalarY_dtoi((*i).valor);
      dc.MoveTo((*i).e==eixo_x?TPoint(cota,0):TPoint(0,cota));
      dc.LineTo((*i).e==eixo_x?TPoint(cota,GetWindowRect().Height()):TPoint(GetWindowRect().Width(),cota));
    };
}

void
TDelgadaView::DesenharSecao (TDC& dc)
{
  vector<pair<template_point<double>,template_point<double> > > v; //trechos
  vector<double> esp;  //espessuras

  if (!delgada_doc.DadosParaDesenharSecao (v,esp)) return;
  CHECK(v.size() == esp.size());

  dc.SelectObject (TPen(TColor::Black,4,PS_SOLID));
  dc.SelectObject(TFont((char far*)0,14,6,0,0,FW_THIN,DEFAULT_PITCH|FF_SWISS,
                    false,false,false,1,OUT_TT_ONLY_PRECIS));
  dc.SetTextAlign(TA_BASELINE|TA_CENTER);

  for (int i = 0; i < v.size(); i++){
    TPoint pi = Escalar(v[i].first);
    TPoint pf = Escalar(v[i].second);
    if (que_mostrar & MOSTRAR_BAGUNCADO){
      int base1 = 10000*sin(v[i].first.x*v[i].first.y+random_source);  //Nmero qualquer,
      int base2 = 10000*sin(v[i].second.x*v[i].second.y+random_source);//escolhido a zio.
      double ang1 = base1%360; ang1 *= M_PI/180;
      double ang2 = base2%360; ang2 *= M_PI/180;
      pi += TSize(25*cos(ang1),25*sin(ang1));
      pf += TSize(25*cos(ang2),25*sin(ang2));
    };
    dc.MoveTo(pi);
    dc.LineTo(pf);
    if (que_mostrar & MOSTRAR_ESPESSURA){
      char buffer[100];
      ostrstream ostr (buffer,100);
      ostr << esp[i] << ends;
      dc.TextOut(TPoint((pi.x+pf.x)/2,(pi.y+pf.y)/2),buffer);
    };
  };
}

void
TDelgadaView::DesenharTodasAsDistancias (TDC& dc)
{
  dc.SelectObject (TPen(TColor::Gray,1,PS_SOLID));
  dc.SelectObject(TFont((char far*)0,14,6,0,0,FW_LIGHT,DEFAULT_PITCH|FF_SWISS,
                    false,false,false,1,OUT_TT_ONLY_PRECIS));
  dc.SelectObject (TBrush(TColor::Gray));
  dc.SetTextAlign(TA_BASELINE|TA_CENTER);

  vector<DistanciaEntreCotas>::const_iterator i;
  for (i=delgada_doc.distancias.begin();i!=delgada_doc.distancias.end();i++)
    if (delgada_doc.DistanciaExiste(*i)){
      eixo e = delgada_doc.cotas[(*i).c1].e;
      double donded = delgada_doc.cotas[(*i).c1].valor;
      double pronded = delgada_doc.cotas[(*i).c2].valor;
      double ortd = (*i).posicao_ortogonal;
      int donde = e==eixo_x?EscalarX_dtoi(donded):EscalarY_dtoi(donded);
      int pronde = e==eixo_x?EscalarX_dtoi(pronded):EscalarY_dtoi(pronded);
      int ort = e==eixo_x?EscalarY_dtoi(ortd):EscalarX_dtoi(ortd);
      TPoint inicial = e==eixo_x?TPoint(donde,ort):TPoint(ort,donde);
      TPoint final = e==eixo_x?TPoint(pronde,ort):TPoint(ort,pronde);
      dc.MoveTo(inicial);
      dc.LineTo(final);
      dc.Ellipse (inicial-TSize(2,2),inicial+TSize(3,3));
      dc.Ellipse (final-TSize(2,2),final+TSize(3,3));
      if (que_mostrar & MOSTRAR_DISTANCIASORIGEM) {
        char buffer[100];
        ostrstream ostr (buffer,100);
        ostr << fabs(donded-pronded) << ends;
        dc.ExtTextOut(TPoint((inicial.x+final.x)/2,(inicial.y+final.y)/2),0,NULL,buffer,-1);
      };
    };
}

void
TDelgadaView::DesenharPreviewR2_NOT (TDC& dc)
{
  dc.SelectObject (TPen(TColor(255,75,75),1,PS_SOLID));
  dc.SetROP2(R2_NOTXORPEN);

  if (que_fazer & SECAO && ha_last_pmouse_d){
    dc.MoveTo(Escalar(last_pmouse_d));
    dc.LineTo(Escalar(pmouse_d));
  };
  if (que_fazer & DISTANCIAS && ha_last_pmouse_d){
    dc.MoveTo(Escalar(last_pmouse_d));
    dc.LineTo(Escalar(eixo_distancia==eixo_x?template_point<double>(pmouse_d.x,last_pmouse_d.y):
                              template_point<double>(last_pmouse_d.x,pmouse_d.y)));
  };
  if (que_fazer & ZOOM && ha_last_pmouse_i){
    dc.Rectangle(last_pmouse_i,pmouse_i);
  };
}
void TDelgadaView::Desenhar_CG_Eixos_K_Circulo_D_Origem (TDC& dc)
{
  template_point<double> CGd,Kd,Dd,Od;
  double teta,Rd;
  TPoint CG,K,D,O;
  int R,sin100,cos100;

  bool temCG = que_mostrar&MOSTRAR_CGEIXOS && delgada_doc.GetCG(CGd);
  bool temK = que_mostrar&MOSTRAR_KCIRCULO && delgada_doc.GetK(Kd);
  bool temD = que_mostrar&MOSTRAR_D && delgada_doc.GetD(Dd);
  bool temO = que_mostrar&MOSTRAR_DISTANCIASORIGEM && delgada_doc.GetOrigem(Od);
  bool temEixo = temCG && delgada_doc.GetTeta_xprincipal(teta);
  bool temR = temK && delgada_doc.GetR(Rd);

  if (temCG) CG = Escalar (CGd);
  if (temK) K = Escalar (Kd);
  if (temD) D = Escalar (Dd);
  if (temO) O = Escalar (Od);
  if (temEixo) sin100 = 100*sin(teta);
  if (temEixo) cos100 = 100*cos(teta);
  if (temR) R = EscalarDistancia_dtoi (Rd);

  dc.SelectObject(TPen(TColor::Black,1,PS_DOT));
  dc.SelectStockObject(NULL_BRUSH);

  if (temEixo){
    dc.MoveTo (CG); dc.LineTo (CG+TSize(cos100,-sin100));
    dc.MoveTo (CG); dc.LineTo (CG+TSize(-sin100,-cos100));
  };
  dc.SelectObject(TPen(TColor::Gray,1,PS_SOLID));
  if (temR){
    dc.Ellipse (K.OffsetBy(-R/2,-R/2),K.OffsetBy(R/2,R/2));
  };

  dc.SelectObject(TPen(TColor::Black));
  dc.SelectObject(TBrush(TColor::Black));
  dc.SelectObject(TFont((char far*)0,18,7,0,0,FW_SEMIBOLD,DEFAULT_PITCH|FF_ROMAN,
                   false,false,false,1,OUT_TT_ONLY_PRECIS));
  dc.SetTextAlign(TA_TOP|TA_LEFT);

  if (temO){
    dc.Ellipse (O-TSize(2,2),O+TSize(3,3));
    dc.TextOut(O+TSize(2,2),"(0,0)");
  };
  if (temCG){
    dc.Ellipse (CG-TSize(2,2),CG+TSize(3,3));
    dc.TextOut(CG+TSize(2,2),"CG");
  };
  if (temEixo){
    dc.TextOut(CG+TSize(cos100,-sin100)+TSize(2,2),"x");
    dc.TextOut(CG+TSize(-sin100,-cos100)+TSize(2,2),"y");
  };
  if (temK){
    dc.Ellipse (K-TSize(2,2),K+TSize(3,3));
    dc.TextOut(K+TSize(2,2),"K");
  };
  if (temD){
    dc.Ellipse (D-TSize(2,2),D+TSize(3,3));
    dc.TextOut(D+TSize(2,2),"D");
  };
};

void TDelgadaView::Escrever_w (TDC& dc)
{
  if (!(que_mostrar & MOSTRAR_SETORIAL)) return;

  dc.SelectObject (TFont((char far*)0,15,6,0,0,FW_MEDIUM,DEFAULT_PITCH|FF_SWISS,
                                     false,false,false,1,OUT_TT_ONLY_PRECIS,
                                     CLIP_DEFAULT_PRECIS,PROOF_QUALITY));
  dc.SetTextAlign(TA_BASELINE|TA_CENTER);

  vector<double> ws;
  vector<template_point<double> >pos;
  if (!delgada_doc.GetWs(ws) || !delgada_doc.GetPosicao_dos_ws(pos))
    return;

  CHECK (ws.size() == pos.size());

  for (int i = 0; i < ws.size(); i++) {
    char buffer[100];
    ostrstream ostr (buffer,100);
    ostr << ws[i] << ends;
    dc.TextOut(Escalar(pos[i]),buffer);
  };
}
void TDelgadaView::Paint(TDC& dc, bool erase, TRect& rect)
{
  DesenharTodasAsCotas (dc);
  DesenharSecao (dc);
  DesenharTodasAsDistancias (dc);
  Desenhar_CG_Eixos_K_Circulo_D_Origem (dc);
  Escrever_w (dc);

  DesenharPreviewR2_NOT(dc);
}

bool TDelgadaView::VnRevert(bool)
{
  double aux;
  template_point<double> paux1,paux2;
  vector<double> vaux;

  ha_last_pmouse_d = false;

  if (que_fazer & MAOZINHA && GetCapture()==GetHandle())
    ReleaseCapture();
  que_fazer_enable = que_mostrar_enable = ~0;

  if (delgada_doc.NaoHaDados()) que_fazer_enable &= ~(DISTANCIAS|BORRACHA|MAOZINHA|ZOOM|METAFILE);
  if (!delgada_doc.GetCG(paux1)) que_mostrar_enable &= ~MOSTRAR_CGEIXOS;
  if (!delgada_doc.GetK(paux1)) que_mostrar_enable &= ~MOSTRAR_KCIRCULO;
  if (!delgada_doc.GetWs (vaux)) que_mostrar_enable &= ~MOSTRAR_SETORIAL;
  if (!delgada_doc.GetD(paux1)) que_mostrar_enable &= ~MOSTRAR_D;
  if (delgada_doc.NaoHaTrechos()) que_mostrar_enable &= ~MOSTRAR_BAGUNCADO;

  que_fazer &= que_fazer_enable;
  que_mostrar &= que_mostrar_enable;

  if (!(que_fazer & (DISTANCIAS|BORRACHA|ORIGEM)))
    que_fazer |= SECAO;

  char buffer[1000];
  char* i_buff = buffer;
  ostrstream ostr (buffer,1000);

  ostr << "rea= "; if (delgada_doc.GetArea(aux)) ostr << aux; ostr << ends;
  valores_texto[CM_TEXTOAREA] = i_buff; while (*(i_buff++));

  ostr << "CG= ";
  if (delgada_doc.GetCG(paux1) && delgada_doc.GetOrigem(paux2))
    ostr << "(" << (paux1.x - paux2.x) << " , " << (paux1.y - paux2.y) << ")";
  ostr << ends;
  valores_texto[CM_TEXTOCG] = i_buff; while (*(i_buff++));

  ostr << "D= ";
  if (delgada_doc.GetD(paux1) && delgada_doc.GetOrigem(paux2))
    ostr << "(" << (paux1.x - paux2.x) << " , " << (paux1.y - paux2.y) << ")";
  ostr << ends;
  valores_texto[CM_TEXTOD] = i_buff; while (*(i_buff++));

  ostr << "Iw= "; if (delgada_doc.GetIw(aux)) ostr << aux; ostr << ends;
  valores_texto[CM_TEXTOIW] = i_buff; while (*(i_buff++));

  ostr << "Ix= "; if (delgada_doc.GetMomentosPrincipais(paux1)) ostr << paux1.x; ostr << ends;
  valores_texto[CM_TEXTOIX] = i_buff; while (*(i_buff++));

  ostr << "Iy= "; if (delgada_doc.GetMomentosPrincipais(paux1)) ostr << paux1.y; ostr << ends;
  valores_texto[CM_TEXTOIY] = i_buff; while (*(i_buff++));

  ostr << "Jt= "; if (delgada_doc.GetJt(aux)) ostr << aux; ostr << ends;
  valores_texto[CM_TEXTOJT] = i_buff; while (*(i_buff++));

  ostr << "K= ";
  if (delgada_doc.GetK(paux1) && delgada_doc.GetOrigem(paux2))
    ostr << "(" << (paux1.x - paux2.x) << " , " << (paux1.y - paux2.y) << ")";
  ostr << ends;
  valores_texto[CM_TEXTOK] = i_buff; while (*(i_buff++));

  ostr << "R= "; if (delgada_doc.GetR(aux)) ostr << aux; ostr << ends;
  valores_texto[CM_TEXTOR] = i_buff; while (*(i_buff++));

  ostr << "tetax= ";
  if (delgada_doc.GetTeta_xprincipal(aux)){
    modf (aux*100*180/M_PI,&aux); // *100 para manter duas casas decimais.
    ostr << aux/100 << "" ;
  };
  ostr << ends;
  valores_texto[CM_TEXTOTETAX] = i_buff;


  Invalidate();
  return true;
}

bool TDelgadaView::VnVouEscalar(double* quanto)
{
  x_no_zero *= *quanto;
  y_no_zero *= *quanto;
  dreal_djanela *= *quanto;
  return true;
}

void TDelgadaView::EvCaptureChanged(HWND)
{
  que_fazer &= ~MAOZINHA;
}

void TDelgadaView::EvLButtonDown(uint, TPoint&)
{
  DesenharPreviewR2_NOT (TClientDC(*this));
  bool keep_last_pmouse_d = false;
  TDo_Undo* nova_action = 0;


  if (que_fazer & MAOZINHA){
    last_pmouse_i = pmouse_i;
    SetCapture();
  } else if (que_fazer & ZOOM){
    if (ha_last_pmouse_i) {
      if (last_pmouse_i != pmouse_i){
        TZoomTipo zt(this);
        if (zt.Execute()==IDOK){
          TRect where_to_fit = GetClientRect();
          template_rect<double> what_to_fit = Escalar(where_to_fit);
          switch (zt.Tipo()){
            case TZoomTipo::comprimir:
              where_to_fit = TRect(last_pmouse_i,pmouse_i);
              break;
            case TZoomTipo::expandir:
              what_to_fit = Escalar(TRect(last_pmouse_i,pmouse_i));
              break;
            case TZoomTipo::fit_all:
              where_to_fit = TRect(last_pmouse_i,pmouse_i);
              double minx,maxx,miny,maxy;
              if (!delgada_doc.LimitesDaSecao(minx,maxx,eixo_x)) minx = maxx = 0;
              if (!delgada_doc.LimitesDaSecao(miny,maxy,eixo_y)) miny = maxy = 0;
              what_to_fit = template_rect<double> (minx,miny,maxx,maxy);
              break;
          };
          Fit (where_to_fit,what_to_fit);
        };
      };
      que_fazer &= ~ZOOM;
      Invalidate();
    } else {
      ha_last_pmouse_i = true;
      last_pmouse_i = pmouse_i;
    };
  } else if (que_fazer & ORIGEM){
    nova_action = new TDo_Undo_Origem (delgada_doc,pmouse_d);
    if (!(que_mostrar & MOSTRAR_DISTANCIASORIGEM)) Invalidate();
    que_mostrar |= MOSTRAR_DISTANCIASORIGEM;
  } else if (que_fazer & SECAO){
    if (ha_trecho){
      if (!(que_mostrar & MOSTRAR_ESPESSURA)) Invalidate();
      que_mostrar |= MOSTRAR_ESPESSURA;
      TLerEspessura LerEspessura(this);
      LerEspessura.SetDados(delgada_doc.trechos[i_trecho].espessura);
      if (LerEspessura.Execute()==IDOK){
        double nova_espessura;
        bool toda_a_secao;
        LerEspessura.GetDados(nova_espessura,toda_a_secao);
        nova_action = toda_a_secao? new TDo_Undo_ModificarEspessura(delgada_doc,nova_espessura)
                                   :new TDo_Undo_ModificarEspessura(delgada_doc,i_trecho,nova_espessura);
      };
    } else {
      if (ha_last_pmouse_d && pmouse_d != last_pmouse_d)
        nova_action = new TDo_Undo_IntroduzirTrecho(delgada_doc,last_pmouse_d,pmouse_d);
      else if (!ha_cota_x || !ha_cota_y)
        nova_action = new TDo_Undo_IntroduzirVertices(delgada_doc,pmouse_d);
      keep_last_pmouse_d = true;
      last_pmouse_d = pmouse_d;
    };
  } else if (que_fazer & DISTANCIAS){
    if (ha_distancia){
      if (!(que_mostrar & MOSTRAR_DISTANCIASORIGEM)) Invalidate();
      que_mostrar |= MOSTRAR_DISTANCIASORIGEM;
      double distancia = fabs (delgada_doc.cotas[delgada_doc.distancias[i_distancia].c1].valor
                            - delgada_doc.cotas[delgada_doc.distancias[i_distancia].c2].valor);
      TLerDistancia LerDistancia (this);
      LerDistancia.SetDados (distancia,!delgada_doc.JaFoiEscalado);
      if (!delgada_doc.JaFoiEscalado) LerDistancia.TravarEscalar();
      if (LerDistancia.Execute()==IDOK){
        double nova_distancia;
        bool escalou;
        LerDistancia.GetDados(nova_distancia,escalou);
        if (escalou){
          nova_action = new TDo_Undo_EscalarCotas(delgada_doc,nova_distancia/distancia);
        } else {
          vector<bool> cotas_que_mudam;
          if (delgada_doc.CotasQueMudariamSeMudasseDistancia (i_distancia,cotas_que_mudam))
            nova_action = new TDo_Undo_ModificarCotas(delgada_doc,cotas_que_mudam,nova_distancia-distancia);
          else
            MessageBox("A distncia selecionada est superdeterminada e no pode ser alterada.", "Modificar distncia", MB_OK);
        };
      };
    } else {
      if (!ha_last_pmouse_d && ha_cota){
        eixo_distancia = delgada_doc.cotas[i_cota].e;
        last_pmouse_d = pmouse_d;
        keep_last_pmouse_d = true;
        prum_lado_ou_pro_outro = 0;
      };
      if (ha_last_pmouse_d){
        double c1 = eixo_distancia==eixo_x?last_pmouse_d.x:last_pmouse_d.y;
        double c2 = eixo_distancia==eixo_x?pmouse_d.x:pmouse_d.y;
        int plopo = c2>c1?1:c1>c2?-1:0;
        if (plopo && (plopo==prum_lado_ou_pro_outro || !prum_lado_ou_pro_outro)){
          double ortog = eixo_distancia==eixo_x?last_pmouse_d.y:last_pmouse_d.x;
          nova_action = new TDo_Undo_IntroduzirDistancia (delgada_doc,c1,c2,ortog,eixo_distancia);
          keep_last_pmouse_d = true;
          last_pmouse_d = eixo_distancia==eixo_x?template_point<double>(c2,ortog):template_point<double>(ortog,c2);
          prum_lado_ou_pro_outro = plopo;
        };
      };
    };
  } else if (que_fazer & BORRACHA && (ha_cota || ha_trecho || ha_distancia)){
    nova_action = new TDo_Undo_Apagar (delgada_doc,
                  ha_distancia?TDo_Undo_Apagar::distancia:ha_trecho?TDo_Undo_Apagar::trecho:TDo_Undo_Apagar::cota,
                  ha_distancia?i_distancia:ha_trecho?i_trecho:i_cota);
    keep_last_pmouse_d = true;
  };

  if (nova_action){
    delgada_doc.AddDoUndo (nova_action);
    delgada_doc.Do();
  };

  if (keep_last_pmouse_d) ha_last_pmouse_d = true; //hack para evitar que VnRevert interrompa alguma ao.
  DesenharPreviewR2_NOT (TClientDC(*this));
}

void TDelgadaView::EvMouseMove(uint modKeys, TPoint& point)
{
  DesenharPreviewR2_NOT (TClientDC(*this));

  pmouse_i = point;
  pmouse_d = Escalar(point);
  ha_cota = ha_cota_x = ha_cota_y = ha_trecho = ha_distancia = false;

  int i_aux;
  double dist1,dist2;
  double maxdist = EscalarDistancia_itod(distancia_maxima_entre_ponto_e_clique_do_mouse);

  if (delgada_doc.CotaNearestPonto(pmouse_d.x,i_cota,dist1,eixo_x) && dist1<=maxdist){
    pmouse_d.x = delgada_doc.cotas[i_cota].valor;
    ha_cota_x = true;
  };
  if (delgada_doc.CotaNearestPonto(pmouse_d.y,i_aux,dist2,eixo_y) && dist2<=maxdist){
    pmouse_d.y = delgada_doc.cotas[i_aux].valor;
    ha_cota_y = true;
    if (!ha_cota_x || dist2<dist1) i_cota = i_aux;
  };
  ha_cota = ha_cota_x || ha_cota_y;
  if (delgada_doc.TrechoDeSecaoNearestPonto (pmouse_d,i_trecho,dist1) && dist1<=maxdist)
    ha_trecho = true;
  if (delgada_doc.LinhaDeDistanciaNearestPonto (pmouse_d,i_distancia,dist1) && dist1<=maxdist)
    ha_distancia = true;

  DesenharPreviewR2_NOT (TClientDC(*this));

  if (que_fazer & MAOZINHA && GetCapture()==GetHandle()){
    x_no_zero -= (pmouse_i.x - last_pmouse_i.x) * dreal_djanela;
    y_no_zero += (pmouse_i.y - last_pmouse_i.y) * dreal_djanela;
    last_pmouse_i = pmouse_i;
    Invalidate();
    UpdateWindow();
  };
  if (que_fazer & MAOZINHA)
    SetCursor (GetApplication(),GetCapture()==GetHandle()?IDC_MAOFECHADA:IDC_MAOABERTA);
  else if (que_fazer & ZOOM)
    SetCursor (0,IDC_CROSS);
  else if ((que_fazer & SECAO || que_fazer & BORRACHA) && ha_trecho)
    SetCursor (GetApplication(),IDC_SECAO);
  else if ((que_fazer & DISTANCIAS || que_fazer & BORRACHA) && ha_distancia)
    SetCursor (GetApplication(),IDC_DISTANCIA);
  else if (ha_cota)
    SetCursor (GetApplication(),IDC_COTA);
  else
    SetCursor (0,IDC_ARROW);
}


void TDelgadaView::EvRButtonDown(uint modKeys, TPoint& point)
{
  CmComandotesoura();
}

void TDelgadaView::EvLButtonUp(uint modKeys, TPoint& point)
{
  if (que_fazer & MAOZINHA && GetCapture()==GetHandle())
    ReleaseCapture();
}


void TDelgadaView::SetupWindow()
{
  TWindowView::SetupWindow();
  VnRevert (false);
}


void TDelgadaView::CeTextoarea(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOAREA].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextocg(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOCG].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextod(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOD].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextoiw(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOIW].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextoix(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOIX].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextoiy(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOIY].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextojt(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOJT].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextok(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOK].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextor(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOR].c_str());
  tce.Enable();
}


void TDelgadaView::CeTextotetax(TCommandEnabler &tce)
{
  tce.SetText (valores_texto[CM_TEXTOTETAX].c_str());
  tce.Enable();
}

void TDelgadaView::CeTextofechada(TCommandEnabler &tce)
{
  tce.Enable(delgada_doc.SecaoFechada());
}


void TDelgadaView::CeTextodescontinua(TCommandEnabler &tce)
{
  tce.Enable(delgada_doc.SecaoDescontinua());
}

void TDelgadaView::CeMostrarcgeixo(TCommandEnabler &tce)
{
  tce.Enable(que_mostrar_enable & MOSTRAR_CGEIXOS);
  tce.SetCheck(que_mostrar & MOSTRAR_CGEIXOS? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TDelgadaView::CeMostrard(TCommandEnabler &tce)
{
  tce.Enable(que_mostrar_enable & MOSTRAR_D);
  tce.SetCheck(que_mostrar & MOSTRAR_D? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TDelgadaView::CeMostrardistorig(TCommandEnabler &tce)
{
  tce.Enable(que_mostrar_enable & MOSTRAR_DISTANCIASORIGEM);
  tce.SetCheck(que_mostrar & MOSTRAR_DISTANCIASORIGEM? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TDelgadaView::CeMostrarespessuras(TCommandEnabler &tce)
{
  tce.Enable(true);
  tce.SetCheck(que_mostrar & MOSTRAR_ESPESSURA? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TDelgadaView::CeMostrarbaguncado(TCommandEnabler &tce)
{
  tce.Enable(que_mostrar_enable & MOSTRAR_BAGUNCADO);
  tce.SetCheck(que_mostrar & MOSTRAR_BAGUNCADO? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TDelgadaView::CeMostrarkcirculo(TCommandEnabler &tce)
{
  tce.Enable(que_mostrar_enable & MOSTRAR_KCIRCULO);
  tce.SetCheck(que_mostrar & MOSTRAR_KCIRCULO? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TDelgadaView::CeMostrarsetorial(TCommandEnabler &tce)
{
  tce.Enable(que_mostrar_enable & MOSTRAR_SETORIAL);
  tce.SetCheck(que_mostrar & MOSTRAR_SETORIAL? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TDelgadaView::CmComandosmostrar (WPARAM wp)
{
  switch (wp) {
    case CM_MOSTRARCGEIXO: que_mostrar ^= MOSTRAR_CGEIXOS; break;
    case CM_MOSTRARD: que_mostrar ^= MOSTRAR_D; break;
    case CM_MOSTRARDISTORIG: que_mostrar ^= MOSTRAR_DISTANCIASORIGEM; break;
    case CM_MOSTRARESPESSURAS: que_mostrar ^= MOSTRAR_ESPESSURA; break;
    case CM_MOSTRARKCIRCULO: que_mostrar ^= MOSTRAR_KCIRCULO; break;
    case CM_MOSTRARSETORIAL: que_mostrar ^= MOSTRAR_SETORIAL; break;
    case CM_MOSTRARBAGUNCADO:
      que_mostrar ^= MOSTRAR_BAGUNCADO;
      time_t t;
      random_source = time(&t);
      break;
  };
  Invalidate ();
}


void TDelgadaView::CeComandoredo(TCommandEnabler &tce)
{
  tce.Enable(delgada_doc.CanDo());
}


void TDelgadaView::CmComandoredo()
{
  delgada_doc.Do();
}


void TDelgadaView::CeComandoundo(TCommandEnabler &tce)
{
  tce.Enable(delgada_doc.CanUndo());
}


void TDelgadaView::CmComandoundo()
{
  delgada_doc.Undo();
}


void TDelgadaView::CeComandoborracha(TCommandEnabler &tce)
{
  tce.Enable (que_fazer_enable & BORRACHA);
  tce.SetCheck(que_fazer&BORRACHA?TCommandEnabler::Checked:TCommandEnabler::Unchecked);
}


void TDelgadaView::CmComandoborracha()
{
  if (que_fazer & BORRACHA) return;

  TBorrachaDialog BorrachaDialog (this, IDD_BORRACHA);
  BorrachaDialog.SetDados(!delgada_doc.NaoHaTrechos(),!delgada_doc.NaoHaDistancias());
  if (BorrachaDialog.Execute()==IDOK){
    switch (BorrachaDialog.Escolha()){
      case TBorrachaDialog::tudo:
        delgada_doc.AddDoUndo(new TDo_Undo_ApagarCompletamente(delgada_doc,TDo_Undo_ApagarCompletamente::tudo));
        delgada_doc.Do();
        break;
      case TBorrachaDialog::secao:
        delgada_doc.AddDoUndo(new TDo_Undo_ApagarCompletamente(delgada_doc,TDo_Undo_ApagarCompletamente::secao));
        delgada_doc.Do();
        break;
      case TBorrachaDialog::distancias:
        delgada_doc.AddDoUndo(new TDo_Undo_ApagarCompletamente(delgada_doc,TDo_Undo_ApagarCompletamente::distancias));
        delgada_doc.Do();
        break;
      case TBorrachaDialog::itens:
        DesenharPreviewR2_NOT (TClientDC(*this));
        que_fazer &= ~EDITANDO_DADOS;
        que_fazer |= BORRACHA;
        DesenharPreviewR2_NOT (TClientDC(*this));
        break;
    };
  };
}


void TDelgadaView::CeComandodistancias(TCommandEnabler &tce)
{
  tce.Enable(que_fazer_enable & DISTANCIAS);
  tce.SetCheck(que_fazer & DISTANCIAS?TCommandEnabler::Checked:TCommandEnabler::Unchecked);
}


void TDelgadaView::CmComandodistancias()
{
  if (que_fazer & DISTANCIAS) return;

  DesenharPreviewR2_NOT (TClientDC(*this));
  que_fazer &= ~EDITANDO_DADOS;
  que_fazer |= DISTANCIAS;
  ha_last_pmouse_d = false;
  DesenharPreviewR2_NOT (TClientDC(*this));
}


void TDelgadaView::CeComandomaozinha(TCommandEnabler &tce)
{
  tce.Enable (que_fazer_enable & MAOZINHA);
  tce.SetCheck(que_fazer & MAOZINHA?TCommandEnabler::Checked:TCommandEnabler::Unchecked);
}


void TDelgadaView::CmComandomaozinha()
{
  DesenharPreviewR2_NOT (TClientDC(*this));
  que_fazer &= ~ZOOM;
  que_fazer ^= MAOZINHA;
  DesenharPreviewR2_NOT (TClientDC(*this));
}


void TDelgadaView::CeComandosecao(TCommandEnabler &tce)
{
  tce.Enable(que_fazer_enable & SECAO);
  tce.SetCheck(que_fazer & SECAO?TCommandEnabler::Checked:TCommandEnabler::Unchecked);
}


void TDelgadaView::CmComandosecao()
{
  if (que_fazer & SECAO) return;
  DesenharPreviewR2_NOT (TClientDC(*this));
  que_fazer &= ~EDITANDO_DADOS;
  que_fazer |= SECAO;
  ha_last_pmouse_d = false;
  DesenharPreviewR2_NOT (TClientDC(*this));
}


void TDelgadaView::CeComandotesoura(TCommandEnabler &tce)
{
  tce.Enable (que_fazer & (SECAO|DISTANCIAS) && ha_last_pmouse_d
              || que_fazer & ZOOM && ha_last_pmouse_i);
}


void TDelgadaView::CmComandotesoura()
{
  DesenharPreviewR2_NOT (TClientDC(*this));
  if (que_fazer & ZOOM && ha_last_pmouse_i)
    que_fazer &= ~ZOOM;
  else
    ha_last_pmouse_d = false;
  DesenharPreviewR2_NOT (TClientDC(*this));
}


void TDelgadaView::CeComandozoom(TCommandEnabler &tce)
{
  tce.Enable (que_fazer_enable & ZOOM);
  tce.SetCheck(que_fazer & ZOOM?TCommandEnabler::Checked:TCommandEnabler::Unchecked);
}


void TDelgadaView::CmComandozoom()
{
  DesenharPreviewR2_NOT (TClientDC(*this));
  que_fazer &= ~MAOZINHA;
  ha_last_pmouse_i = false;
  que_fazer ^= ZOOM;
  DesenharPreviewR2_NOT (TClientDC(*this));
}


void TDelgadaView::CeComandoorigem(TCommandEnabler &tce)
{
  tce.Enable(que_fazer_enable & ORIGEM);
  tce.SetCheck(que_fazer & ORIGEM?TCommandEnabler::Checked:TCommandEnabler::Unchecked);
}


void TDelgadaView::CmComandoorigem()
{
  DesenharPreviewR2_NOT (TClientDC(*this));
  que_fazer &= ~EDITANDO_DADOS;
  que_fazer |= ORIGEM;
  DesenharPreviewR2_NOT (TClientDC(*this));
}


void TDelgadaView::CeComandometafile(TCommandEnabler &tce)
{
  tce.Enable(que_fazer_enable & METAFILE);
}


void TDelgadaView::CmComandometafile()
{
  if (MessageBox ("Copiar imagem da seo para o Clipboard?",
                   GetViewName(),MB_OKCANCEL)==IDOK){
    TClipboard clipboard(*this);
    if (clipboard.EmptyClipboard()) {
      TMetaFileDC mfDC;
      mfDC.SetWindowExt(GetClientRect().Size());
      mfDC.SetWindowOrg(0, 0);
      Paint (mfDC,false,GetClientRect());
      TMetaFilePict mf(mfDC.Close(), AutoDelete);
      mf.ToClipboard(clipboard,MM_TEXT,GetClientRect().Size());
    } else
      MessageBox ("No consegui esvaziar o Clipboard.",GetViewName(),MB_OK);
  };
}


void TDelgadaView::EvSize(uint sizeType, TSize& size)
{
  if (sizeType!=SIZE_MAXIMIZED && sizeType!=SIZE_RESTORED) return;

  TRect areacliente = GetClientRect();
  if (areacliente.IsEmpty()) return;

  if (NotSizedYet) {
    double minx,maxx,miny,maxy;
    if (!delgada_doc.LimitesDaSecao(minx,maxx,eixo_x)) minx = maxx = 0;
    if (!delgada_doc.LimitesDaSecao(miny,maxy,eixo_y)) miny = maxy = 0;
    template_rect<double> r (minx,miny,maxx,maxy);
    r.Inflate (r.Width()/20,r.Height()/20);
    Fit (areacliente,r);
    Invalidate();
    NotSizedYet = false;
  };
}

