//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//

#include <owl/pch.h>
#include <owl/validate.h>

#include "dialogos.h"
#include "dagoapp.rh"

namespace{
  class TDoubleValidator : public TValidator {
    public:
      TDoubleValidator(double inf, double sup, const string& campo_)
        :limite_inferior(inf),limite_superior(sup),campo(campo_){};
      virtual bool IsValid(const char far* input);
      virtual void Error(TWindow* owner);
    protected:
      string campo;
      double limite_inferior, limite_superior;
  };
  bool TDoubleValidator::IsValid(const char far* input)
  {
    size_t sz = strlen (input);
    char* buffer = new char[sz+1];
    strcpy (buffer,input);
    bool ret = false;
    if (sz>0){
      istrstream istr (buffer);
      double valor;
      istr >> valor;
      if (istr.tellg()==sz)
        if (valor >= limite_inferior && valor <= limite_superior)
          ret = true;
    };
    delete[] buffer;
    return ret;
  }
  void TDoubleValidator::Error(TWindow* owner)
  {
    if (!owner) return;
    ostrstream ostr;
    ostr << "Esse campo deve ser preenchido com um valor no intervalo:" << endl;
    ostr << "<" << limite_inferior << "," << limite_superior << ">" << ends;
    char* buffer = ostr.str();
    owner->MessageBox(buffer, campo.c_str());
    delete[] buffer;
  }
};

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TZoomTipo::TZoomTipo(TWindow* parent, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  radio1 = new TRadioButton(this, IDC_RADIOBUTTON1, 0);
  radio2 = new TRadioButton(this, IDC_RADIOBUTTON2, 0);
  radio3 = new TRadioButton(this, IDC_RADIOBUTTON3, 0);
}
TZoomTipo::~TZoomTipo()
{
  Destroy(IDCANCEL);
}
void TZoomTipo::TransferData(TTransferDirection direction)
{
  if (direction==tdSetData){
    radio1->SetCheck(BF_CHECKED);
  }
  else if (direction==tdGetData){
    if (radio1->GetCheck()==BF_CHECKED) tipo = comprimir;
    if (radio2->GetCheck()==BF_CHECKED) tipo = expandir;
    if (radio3->GetCheck()==BF_CHECKED) tipo = fit_all;
  };
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TBorrachaDialog::TBorrachaDialog(TWindow* parent, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  radio1 = new TRadioButton(this, IDC_RADIOBUTTON1, 0);
  radio2 = new TRadioButton(this, IDC_RADIOBUTTON2, 0);
  radio3 = new TRadioButton(this, IDC_RADIOBUTTON3, 0);
  radio4 = new TRadioButton(this, IDC_RADIOBUTTON4, 0);
}
TBorrachaDialog::~TBorrachaDialog()
{
  Destroy(IDCANCEL);
}
void TBorrachaDialog::TransferData(TTransferDirection direction)
{
  if (direction==tdSetData){
    radio4->SetCheck(BF_CHECKED);
    radio2->EnableWindow(temsecao);
    radio3->EnableWindow(temdistancias);
  }
  else if (direction==tdGetData){
    if (radio1->GetCheck()==BF_CHECKED) escolha = tudo;
    if (radio2->GetCheck()==BF_CHECKED) escolha = secao;
    if (radio3->GetCheck()==BF_CHECKED) escolha = distancias;
    if (radio4->GetCheck()==BF_CHECKED) escolha = itens;
  };
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TLerEspessura::TLerEspessura(TWindow* parent, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  CheckBoxTodos = new TCheckBox(this, IDC_CHECKBOX1, 0);
  EditControlEspessura = new TEdit(this, IDC_EDIT1, 255);
  EditControlEspessura->SetValidator(new TDoubleValidator(0.0001,100000,"Nova distncia"));
}
TLerEspessura::~TLerEspessura()
{
  Destroy(IDCANCEL);
}
void TLerEspessura::SetDados (double esp,bool tod)
{
  Espessura = esp;
  Todos = tod;
}
void TLerEspessura::GetDados (double& esp,bool& tod)
{
  esp = Espessura;
  tod = Todos;
}
void TLerEspessura::TransferData(TTransferDirection direction)
{
  char buffer[100];
  if (direction==tdSetData){
    ostrstream ostr (buffer,100);
    ostr << Espessura << ends;
    EditControlEspessura->SetText(buffer);
    CheckBoxTodos->SetCheck(Todos?BF_CHECKED:BF_UNCHECKED);
  }
  else if (direction==tdGetData){
    EditControlEspessura->GetText(buffer,100);
    istrstream istr (buffer,100);
    istr >> Espessura;
    Todos = CheckBoxTodos->GetCheck()==BF_CHECKED;
  };
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TLerDistancia::TLerDistancia(TWindow* parent, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  EditControlDistancia = new TEdit(this, IDC_EDIT1, 255);
  CheckBoxEscalar = new TCheckBox(this, IDC_CHECKBOX1, 0);
  esc_comp = false;
  EditControlDistancia->SetValidator(new TDoubleValidator(0.0001,100000,"Nova distncia"));
}
TLerDistancia::~TLerDistancia()
{
  Destroy(IDCANCEL);
}
void TLerDistancia::SetDados (double dist,bool Escal)
{
  Distancia = dist;
  Escalar = Escal;
}
void TLerDistancia::GetDados (double& dist,bool& Escal)
{
  dist = Distancia;
  Escal = Escalar;
}
void TLerDistancia::TravarEscalar ()
{
  esc_comp = true;
}
void TLerDistancia::TransferData(TTransferDirection direction)
{
  char buffer[100];
  if (direction==tdSetData){
    ostrstream ostr (buffer,100);
    ostr << Distancia << ends;
    EditControlDistancia->SetText(buffer);
    CheckBoxEscalar->SetCheck(Escalar?BF_CHECKED:BF_UNCHECKED);
    CheckBoxEscalar->EnableWindow(!esc_comp);
  }
  else if (direction==tdGetData){
    EditControlDistancia->GetText(buffer,100);
    istrstream istr (buffer,100);
    istr >> Distancia;
    Escalar = CheckBoxEscalar->GetCheck()==BF_CHECKED;
  };
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

char gnu_license[] =
"Licena pblica GNU\r\n"
"GNU GENERAL PUBLIC LICENSE Version 2, June 1991\r\n"
"\r\n"
"\r\n"
"\r\n"
"This is an unofficial translation of the GNU General Public License into Portuguese. It was not published by the Free Software Foundation, and does not legally state the distribution terms for software that uses the GNU GPL -- only the original English text of the GNU GPL does that. However, we hope that this translation will help Portuguese speakers understand the GNU GPL better.\r\n"
"\r\n"
"\r\n"
"Copyright (C) 1989, 1991 Free Software Foundation, Inc. 675 Mass Ave, Cambridge, MA 02139, USA\r\n"
"\r\n"
" permitido a qualquer pessoa copiar e distribuir cpias tal desse documento de licena, sem a implementao de qualquer mudana.\r\n"
"\r\n"
"F.3.1 Introduo\r\n"
"\r\n"
"As licenas de muitos softwares so desenvolvidas para cercear a liberdade de uso, compartilhamento e mudanas. A GNU Licena Pblica Geral ao contrrio, pretende garantir a liberdade de compartilhar e alterar softwares de livre distribuio - tornando-os de livre distribuio tambm para quaisquer usurios. A Licena Pblica Geral aplica-se  maioria dos softwares da Free Software Foundation e a qualquer autor que esteja de acordo de utiliz-la (alguns softwares da FSF so cobertos pela GNU Library General Public License).\r\n"
"Quando nos referimos a softwares de livre distribuio, referimo-nos  liberdade e no ao preo. Nossa Licena Pblica Geral foi criada para garantir a liberdade de distribuio de cpias de softwares de livre distribuio (e cobrar por isso caso seja do interesse do distribuidor), o qual recebeu os cdigos fonte, o qual pode ser alterado ou utilizado em parte em novos programas.\r\n"
"Para assegurar os direitos dos desenvolvedores, algumas restries so feitas, proibindo a todas as pessoas a negao desses direitos ou a solicitao de sua abdicao. Essas restries aplicam-se ainda a certas responsabilidades sobre a distribuio ou modificao do software.\r\n"
"Por exemplo, ao se distribuir cpias de determinado programa, por uma taxa determinada ou gratuitamente,  deve-se informar sobre todos os direitos incidentes sobre esse programa, assegurando-se que os fontes estejam disponveis assim como a Licena Pblica Geral GNU.\r\n"
"A proteo dos direitos envolve dois passos: (1) copyright do software e (2) licena que d permisso legal para cpia, distribuio e/ou modificao do softwares.\r\n"
"Ainda para a proteo da FSF e do autor  importante que todos entendam que no h garantias para softwares de livre distribuio. Caso o software seja modificado por algum e passado adiante, este software no mais refletir o trabalho original do autor no podendo portanto ser garantido por aquele.\r\n"
"Finalmente, qualquer programa de livre distribuio  constantemente ameaado pelas patentes de softwares. Buscamos evitar o perigo de que distribuidores destes programas obtenham patentes individuais, tornado-se seus donos efetivos. Para evitar isso foram feitas declaraes expressas de que qualquer solicitao de patente deve ser feita  permitindo o uso por qualquer indivduo, sem a necessidade de licena de uso.\r\n"
"Os termos e condies precisas para cpia, distribuio e modificao seguem abaixo:\r\n"
"\r\n"
"F.3.2 Licena Pblica Geral GNU\r\n"
"\r\n"
"TERMOS E CONDIES PARA CPIA, DISTRIBUIO E MODIFICAO\r\n"
"\r\n"
"0\r\n"
"\r\n"
"Esta licena se aplica a qualquer programa ou outro trabalho que contenha um aviso colocado pelo detentor dos direitos autorais dizendo que aquele poder ser distribudo nas condies da Licena Pblica Geral. O Programa, abaixo refere-se a qualquer software ou trabalho e a um trabalho baseado em um Programa  e significa tanto o Programa em si como quaisquer trabalhos derivados  de acordo com a lei de direitos autorais, o que significa dizer, um trabalho que contenha o Programa ou uma parte deste, na sua forma original ou com modificaes ou traduzido para uma outra lngua (traduo est includa sem limitaes no termo modificao).\r\n"
"Atividades distintas de cpia, distribuio  e modificao no esto cobertas por esta Licena, estando fora de seu escopo. O ato de executar o Programa no est restringido e a sada do Programa  coberta somente caso seu contedo contenha trabalhos baseados no Programa (independentemente de terem sidos gerados pela execuo do Programa). Se isso  verdadeiro depende das funes executadas pelo Programa.\r\n"
"\r\n"
"1\r\n"
"\r\n"
"O cdigo fonte do Programa, da forma como foi recebido, pode ser copiado e distribudo, em qualquer media, desde que seja providenciada um aviso adequado sobre os copyrights e a negao de garantias, e todos os avisos que se referem  Licena Pblica Geral e  ausncia de garantias estejam inalterados e que qualquer produtos oriundo do Programa esteja acompanhado desta Licena Pblica Geral.\r\n"
" permitida a cobrana de taxas pelo ato fsico de transferncia ou gravao de cpias, e podem ser dadas garantias e suporte em troca da cobrana de valores.\r\n"
"\r\n"
"2\r\n"
"\r\n"
"Pode-se modificar a cpia ou cpias do Programa de qualquer forma que se deseje, ou ainda criar-se um trabalho baseado no Programa, e copi-la e distribuir tais modificaes sob os termos da seo 1 acima e do seguinte:\r\n"
"\r\n"
"1. Deve existir aviso em destaque de que os dados originais foram alterados  nos arquivos e as datas das mudanas;\r\n"
"2. Deve existir aviso de que o trabalho distribudo ou publicado , de forma total ou em parte derivado do Programa  ou de alguma parte sua, e que pode ser licenciado totalmente sem custos para terceiros sob os termos desta Licena.\r\n"
"3. Caso o programa modificado seja executado de forma interativa,  obrigatrio, no incio de sua execuo, apresentar a informao de copyright e da ausncia de garantias (ou de que a garantia corre por conta de terceiros), e que os usurios podem redistribuir o programa sob estas condies, indicando ao usurio como acessar esta Licena na sua ntegra.\r\n"
"Esses requisitos aplicam-se a trabalhos de modificao em geral. Caso algumas sees identificveis no sejam derivadas do Programa, e podem ser consideradas como partes independentes, ento esta Licena e seus Termos no se aplicam quelas sees quando distribudas separadamente. Porm ao distribuir aquelas sees como  parte de um trabalho baseado no Programa, a distribuio como um todo deve conter os termos desta Licena, cujas permisses estendem-se ao trabalho como um todo, e no a cada uma das partes independentemente de quem os tenha desenvolvido.\r\n"
"Mais do que tencionar contestar os direitos sobre o trabalho desenvolvido por algum, esta seo objetiva propiciar a correta distribuio de trabalhos derivados do Programa.\r\n"
"Adicionalmente, a mera adio de outro trabalho ao Programa, porm no baseado nele nem a um trabalho baseado nele,  a um volume de armazenamento ou media de distribuio no obriga a utilizao desta Licena e de seus termos ao trabalho.\r\n"
"\r\n"
"3\r\n"
"\r\n"
"So permitidas a cpia e a distribuio do Programa (ou a um trabalho baseado neste) na forma de cdigo objeto ou executvel de acordo com os termos das Sees 1 e 2 acima, desde que atendido o seguinte:\r\n"
"\r\n"
"1. Esteja acompanhado dos cdigos fonte legveis, os quais devem ser distribudos na forma da Sees 1 e 2 acima, em mdia normalmente utilizada para manuseio de softwares ou\r\n"
"2. Esteja acompanhado de oferta escrita, vlida por, no mnimo 3 anos, de disponibilizar a terceiros, por um custo no superior ao custo do meio fsico de armazenamento , uma cpia completa  dos cdigos fonte em meio magntico, de acordo com as Sees 1 e 2 acima.\r\n"
"3. Esteja acompanhada com a mesma informao recebida em relao  oferta da distribuio do cdigo fonte correspondente. (esta alternativa somente  permitida para distribuies no comerciais e somente se o programa recebido na forma de objeto ou executvel tenha tal oferta, de acordo com a sub-seo 2 acima).\r\n"
"O cdigo fonte de um trabalho  a melhor forma de produzirem-se alteraes naquele trabalho.  Cdigos fontes completos significam todos os fontes de todos os mdulos, alm das definies de interfaces associadas, arquivos, scripts utilizados na compilao e instalao do executvel. Como uma exceo excepcional, o cdigo fonte distribudo poder no incluir alguns componentes que no se encontrem em seu escopo, tais como compilador, kernel, etc... para o SO onde o trabalho seja executado.\r\n"
"Caso a distribuio do executvel ou objeto seja feita atravs de acesso a um determinado ponto, ento oferta equivalente de acesso deve ser feita aos cdigos fonte, mesmo que terceiros no sejam obrigados a copiarem os fontes juntos com os objetos simultaneamente.\r\n"
"\r\n"
"4\r\n"
"\r\n"
"No  permitida a cpia, modificao, sublicenciamento ou distribuio do Programa, exceto sob as condies expressas nesta Licena. Qualquer tentativa de cpia, modificao, sublicenciamento ou distribuio do Programa  proibida, e os direitos descritos nesta Licena cessaro imediatamente. Terceiros que tenham recebido cpias ou direitos na forma desta Licena no tero seus direitos cessados desde que permaneam dentro das clusulas desta Licena.\r\n"
"\r\n"
"5\r\n"
"\r\n"
"No  necessria aceitao formal desta Licena, apesar de que no haver documento ou contrato que garanta permisSo de modificao ou distribuio do Programa ou seus trabalhos derivados. Essas aes So proibidas por lei, caso no se aceitem as condies desta Licena. A modificao ou distribuio do Programa ou qualquer trabalho baseado neste implica na aceitao desta Licena e de todos os termos desta para cpia, distribuio ou modificao do Programa ou trabalhos baseados\r\n"
"neste.\r\n"
"\r\n"
"6\r\n"
"\r\n"
"Cada vez que o Programa seja distribudo (ou qualquer trabalho baseado neste), o recipiente automaticamente recebe uma licena do detentor original dos direitos de cpia, distribuio ou modificao do Programa objeto deste termos e condies. No podem ser impostas outras restries nos recipientes.\r\n"
"\r\n"
"7\r\n"
"\r\n"
"No caso de decises judiciais ou alegaes de uso indevido de patentes ou direitos autorais, restries sejam impostas que contradigam esta Licena, estes no isentam da sua aplicao. Caso no seja possvel distribuir o Programa de forma a garantir simultaneamente as obrigaes desta Licena e outras que sejam necessrias, ento o Programa no poder ser distribudo.\r\n"
"Caso esta Seo seja considerada invlida por qualquer motivo particular ou geral, o seu resultado implicar na invalidao geral desta licena na cpia, modificao, sublicenciamento ou distribuio do Programa ou trabalhos baseados neste.\r\n"
"O propsito desta seo no , de forma alguma, incitar quem quer que seja a infringir direitos reclamados em questes vlidas e procedentes, e sim proteger as premissas do sistema de livre distribuio de software. Muitas pessoas tm feito contribuies generosas ao sistema, na forma de programas, e  necessrio garantir  a consistncia e credibilidade do sistema, cabendo a estes e no a terceiros decidirem  a forma de distribuio dos softwares.\r\n"
"Esta seo pretende tornar claro os motivos que geraram as demais clusulas destas Licena.\r\n"
"\r\n"
"8\r\n"
"\r\n"
"Caso a distribuio do Programa dentro dos termos desta Licena tenha restries em algum Pas, quer por patentes ou direitos autorais, o detentor original dos direitos autorais do Programa sob esta Licena pode adicionar explicitamente limitaes geogrficas de distribuio, excluindo aqueles Pases, fazendo com que a distribuio somente seja possvel nos Pases no excludos.\r\n"
"\r\n"
"9\r\n"
"\r\n"
"A Fundao de Software de Livre Distribuio (FSF - Free Software Foundation) pode publicar verses revisadas ou novas verses desta Licena Pblica Geral de tempos em tempos. Estas novas verses mantero os mesmos objetivos e o esprito da presente verso, podendo variar em detalhes referentes a novas situaes encontradas.\r\n"
"A cada verso  dada um nmero distinto. Caso o Programa especifique um nmero de verso especfico desta Licena a qual tenha em seu contedo a expresso - ou verso mais atualizada-,  possvel optar pelas condies daquela verso ou de qualquer verso mais atualizada publicada pela FSF.\r\n"
"\r\n"
"10\r\n"
"\r\n"
"Caso se deseje incorporar parte do Programa em outros programas de livre distribuio de softwares  necessria autorizao formal do autor. Para softwares que a FSF detenha os direitos autorais, podem ser abertas excees desde que mantido o esprito e objetivos originais desta Licena.\r\n"
"\r\n"
"11\r\n"
"\r\n"
"AUSNCIA DE GARANTIAS\r\n"
"\r\n"
"UMA VEZ QUE O PROGRAMA  LICENCIADO SEM NUS, NO H QUALQUER GARANTIA PARA O PROGRAMA. EXCETO QUANDO TERCEIROS EXPRESSEM-SE FORMALMENTE O PROGRAMA  DISPONIBILIZADO EM SEU FORMATO ORIGINAL, SEM GARANTIAS DE QUALQUER NATUREZA, EXPRESSAS OU IMPLCITAS, INCLUINDO MAS NO LIMITADAS, A GARANTIAS COMERCIAIS  E DO ATENDIMENTO DE DETERMINADO FIM. A QUALIDADE E A PERFORMANCE SO DE RISCO EXCLUSIVO DOS USURIOS, CORRENDO POR SUAS CONTA OS CUSTOS NECESSRIOS A EVENTUAIS ALTERAES, CORREES E REPAROS JULGADOS NECESSRIOS.\r\n"
"EM NENHUMA OCASIO, A MENOS QUE REQUERIDO POR DECISO JUDICIAL OU POR LIVRE VONTADE, O AUTOR OU TERCEIROS QUE TENHAM MODIFICADO O PROGRAMA, SERO RESPONSVEIS POR DANOS OU PREJUZOS PROVENIENTES DO USO OU DA FALTA DE HABILIDADE NA SUA UTILIZAO (INCLUINDO MAS NO LIMITADA A PERDA DE DADOS OU DADOS ERRNEOS), MESMO QUE TENHA SIDO EMITIDO AVISO DE POSSVEIS ERROS OU DANOS.\r\n"
"\r\n"
"FIM DA LICENA\r\n"
"\r\n"
"\r\n"
"F.3.3 Apndice\r\n"
"\r\n"
"Como aplicar estes termos a novos softwares?\r\n"
"\r\n"
"Caso se  tenha desenvolvido um novo programa e se deseje a sua ampla distribuio para o pblico, a melhor forma de consegui-lo  torn-lo um software de livre distribuio, o qual qualquer um possa distribu-lo nas condies desta Licena.\r\n"
"Para tanto basta anexar este aviso ao programa.  aconselhvel indicar ainda no incio de cada arquivo fonte a ausncia de garantias e um apontamento para um arquivo contendo o texto geral desta Licena, como por exemplo:\r\n"
"\r\n"
"<nome do programa e funo> Copyright (C) 199X <Autor>\r\n"
"\r\n"
"Este programa  um software de livre distribuio, que pode ser copiado e distribudo sob os termos da Licena Pblica Geral  GNU, conforme publicada pela Free Software Foundation, verso 2 da licena ou (a critrio do autor) qualquer verso posterior.\r\n"
"Este programa  distribudo na expectativa de ser til aos seus usurios, porm  NO TEM NENHUMA GARANTIA,  EXPLCITAS OU IMPLCITAS , COMERCIAIS OU DE ATENDIMENTO A UMA DETERMINADA FINALIDADE. Consulte a Licena Pblica Geral  GNU para maiores detalhes.\r\n"
"Deve haver uma cpia da Licena Pblica Geral  GNU  junto com este software em ingls ou portugus. Caso no haja escreva para Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\r\n"
"\r\n"
"Autor@mail.com.br\r\n"
"Endereo\r\n"
"\r\n"
"Caso o programa seja interativo, apresente na sua sada um breve aviso quando de seu incio como por exemplo:\r\n"
"\r\n"
"\r\n"
"\r\n"
"Gnomovision verso 69, Copyright 199a Yoyodine\r\n"
"\r\n"
"Softwares  NO POSSUI  NENHUMA  GARANTIA;  para detalhes digite mostre garantia. Este  um software de livre distribuio e voc est autorizado a distribui-lo dentro de certas condies. Digite mostre condio para maiores detalhes.\r\n"
"Os comandos hipotticos mostre garantia e mostre condio apresentaro as partes apropriadas da Licena Pblica Geral GNU. Evidentemente os comandos podem variar ou serem acionado por outras interfaces como clique de mouse, etc...\r\n"
"\r\n"
"\r\n";


SplashDialog::SplashDialog(TWindow* parent, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  gnu_txt = new TEdit(this, IDC_EDIT1, 255);
  version = new TStatic(this, IDC_STATICTEXT3, 255);
}
SplashDialog::~SplashDialog()
{
  Destroy(IDCANCEL);
}
void SplashDialog::SetupWindow()
{
  TDialog::SetupWindow();
  gnu_txt->SetText(gnu_license);
  version->SetText("Version:\n"__DATE__);
}

