//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//

#include <owl/pch.h>

#include <utility>

#include <do_undo_inheriteds.hpp>

using namespace std;

TDo_Undo_Introduzircotas::TDo_Undo_Introduzircotas(TDelgadaDoc& d,double posicao,eixo e)
          : TDo_Undo (d)
{
  cota_x = (e == eixo_x); cota_y = (e == eixo_y);
  pos_x = pos_y = posicao;
}
TDo_Undo_Introduzircotas::TDo_Undo_Introduzircotas(TDelgadaDoc& d,double posicaox,double posicaoy)
          : TDo_Undo (d)
{
  cota_x = cota_y = true;
  pos_x = posicaox; pos_y = posicaoy;
}
void
TDo_Undo_Introduzircotas::ActualDo()
{
  if (cota_x){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = pos_x;
    delgada_doc.cotas.back().e     = eixo_x;
  };
  if (cota_y){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = pos_y;
    delgada_doc.cotas.back().e     = eixo_y;
  };
}
void
TDo_Undo_Introduzircotas::ActualUndo()
{
  if (cota_x) delgada_doc.cotas.pop_back();
  if (cota_y) delgada_doc.cotas.pop_back();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_Introduzirdistancia::TDo_Undo_Introduzirdistancia (TDelgadaDoc& d,int c1_,int c2_, double posicao_)
          : TDo_Undo (d),c1(c1_),c2(c2_),posicao(posicao_)
{
}
void TDo_Undo_Introduzirdistancia::ActualDo()
{
  DistanciaEntreCotas dist;
  dist.valid = true;
  dist.c1 = c1;
  dist.c2 = c2;
  dist.posicao_ortogonal = posicao;

  delgada_doc.distancias.push_back(dist);
}
void TDo_Undo_Introduzirdistancia::ActualUndo()
{
  delgada_doc.distancias.pop_back();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_EscalarCotas::TDo_Undo_EscalarCotas(TDelgadaDoc& d,double escala_)
          : TDo_Undo (d),escala(escala_),cotas(delgada_doc.cotas)
{
  JaFoiEscalado = true;
  for (int i = 0; i < cotas.size(); i++)
    cotas[i].valor *= escala;
}
void TDo_Undo_EscalarCotas::ActualDo()
{
  swap(delgada_doc.JaFoiEscalado,JaFoiEscalado);
  swap(delgada_doc.cotas,cotas);
  for (int i = 0; i < delgada_doc.distancias.size(); i++)
    delgada_doc.distancias[i].posicao_ortogonal *= escala;

  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_EscalarCotas::ActualUndo()
{
  swap(delgada_doc.JaFoiEscalado,JaFoiEscalado);
  swap(delgada_doc.cotas,cotas);
  for (int i = 0; i < delgada_doc.distancias.size(); i++)
    delgada_doc.distancias[i].posicao_ortogonal /= escala;

  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_EscalarCotas::Before(DoingOrUndoing what)
{
  double b = (what==doing)?escala:1/escala;
  delgada_doc.NotifyViews(vnVouEscalar,reinterpret_cast<long>(&b));
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_ModificarCotas::TDo_Undo_ModificarCotas(TDelgadaDoc& d,const vector<bool>& v,double deslocamento_)
          : TDo_Undo(d),vbool(v),deslocamento(deslocamento_)
{
}
void TDo_Undo_ModificarCotas::ActualDo()
{
  SomarNasCotas(deslocamento);
  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_ModificarCotas::ActualUndo()
{
  SomarNasCotas(-deslocamento);
  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_ModificarCotas::SomarNasCotas (const double& d)
{
  CHECK (delgada_doc.cotas.size() == vbool.size());
  for (int i = 0; i < vbool.size(); i++)
    if (vbool[i])
      delgada_doc.cotas[i].valor += d;
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_ModificarEspessura::TDo_Undo_ModificarEspessura(TDelgadaDoc& d,int id_trecho_, double espessura_)
          : TDo_Undo(d),id_trecho(id_trecho_),espessura(espessura_)
{
  todos_os_trechos = false;
}
TDo_Undo_ModificarEspessura::TDo_Undo_ModificarEspessura(TDelgadaDoc& d,double espessura_)
          : TDo_Undo(d),espessura(espessura_)
{
  todos_os_trechos = true;
  for (int i=0;i<delgada_doc.trechos.size();i++)
    espessuras.push_back(delgada_doc.trechos[i].espessura);
}
void TDo_Undo_ModificarEspessura::ActualDo()
{
  if (todos_os_trechos){
    for (int i=0;i<delgada_doc.trechos.size();i++)
      delgada_doc.trechos[i].espessura = espessura;
  }else
    swap (delgada_doc.trechos[id_trecho].espessura,espessura);
  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_ModificarEspessura::ActualUndo()
{
  if (todos_os_trechos){
    for (int i=0;i<delgada_doc.trechos.size();i++)
      delgada_doc.trechos[i].espessura = espessuras[i];
  }else
    swap (delgada_doc.trechos[id_trecho].espessura,espessura);
  delgada_doc.DesatualizarCalculos();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_FixarOrigem::TDo_Undo_FixarOrigem(TDelgadaDoc& d,int cx_,int cy_)
          : TDo_Undo(d),cx(cx_),cy(cy_)
{
}
void TDo_Undo_FixarOrigem::ActualDo()
{
  swap (cx,delgada_doc.cxOrigem);
  swap (cy,delgada_doc.cyOrigem);
}
void TDo_Undo_FixarOrigem::ActualUndo()
{
  ActualDo();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_ApagarCompletamente::TDo_Undo_ApagarCompletamente (TDelgadaDoc& d,partes oque)
           :TDo_Undo(d)
{
  delsecao = oque==secao || oque==tudo;
  deldistancias = oque==distancias || oque==tudo;
  delcotas = oque==tudo;
}
void TDo_Undo_ApagarCompletamente::ActualDo()
{
  if (delsecao) swap (delgada_doc.trechos,t);
  if (deldistancias) swap (delgada_doc.distancias,d);
  if (delcotas) swap (delgada_doc.cotas,c);
  if (delcotas) swap (delgada_doc.cxOrigem,cxOrigem);
  if (delcotas) swap (delgada_doc.cyOrigem,cyOrigem);
  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_ApagarCompletamente::ActualUndo()
{
  ActualDo();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_IntroduzirVertices::TDo_Undo_IntroduzirVertices(TDelgadaDoc& d,template_point<double> posicao)
          : TDo_Undo (d),pos_x(posicao.x),pos_y(posicao.y)
{

  cota_x = cota_y = false;
  for (vector<cota>::const_iterator i=delgada_doc.cotas.begin();i!=delgada_doc.cotas.end();i++)
    if (delgada_doc.CotaExiste(*i)){
      if ((*i).e==eixo_x && (*i).valor==pos_x) cota_x = true;
      if ((*i).e==eixo_y && (*i).valor==pos_y) cota_y = true;
    };
}
void
TDo_Undo_IntroduzirVertices::ActualDo()
{
  if (!cota_x){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = pos_x;
    delgada_doc.cotas.back().e     = eixo_x;
  };
  if (!cota_y){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = pos_y;
    delgada_doc.cotas.back().e     = eixo_y;
  };
}
void
TDo_Undo_IntroduzirVertices::ActualUndo()
{
  if (!cota_x) delgada_doc.cotas.pop_back();
  if (!cota_y) delgada_doc.cotas.pop_back();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

namespace{
  bool MesmasCotas (trecho t1, const trecho& t2)
  {
    if (t1.cx1 == t2.cx1 && t1.cy1 == t2.cy1 && t1.cx2 == t2.cx2 && t1.cy2 == t2.cy2)
      return true;
    swap (t1.cx1,t1.cx2); swap (t1.cy1,t1.cy2);
    if (t1.cx1 == t2.cx1 && t1.cy1 == t2.cy1 && t1.cx2 == t2.cx2 && t1.cy2 == t2.cy2)
      return true;
    return false;
  }
}

TDo_Undo_IntroduzirTrecho::TDo_Undo_IntroduzirTrecho(TDelgadaDoc& d,template_point<double> p1_,template_point<double> p2_)
          : TDo_Undo (d),p1(p1_),p2(p2_)
{
  bc1x = bc1y = bc2x = bc2y = false;
  ja_existia = false;

  for (vector<cota>::const_iterator i=delgada_doc.cotas.begin();i!=delgada_doc.cotas.end();i++)
    if (delgada_doc.CotaExiste(*i)){
      int indice = i - delgada_doc.cotas.begin();
      if ((*i).e==eixo_x && (*i).valor==p1.x) {bc1x = true; c1x = indice;};
      if ((*i).e==eixo_y && (*i).valor==p1.y) {bc1y = true; c1y = indice;};
      if ((*i).e==eixo_x && (*i).valor==p2.x) {bc2x = true; c2x = indice;};
      if ((*i).e==eixo_y && (*i).valor==p2.y) {bc2y = true; c2y = indice;};
    };

  int indice = delgada_doc.cotas.size();
  if (!bc1x) c1x = indice++;
  if (!bc1y) c1y = indice++;
  if (!bc2x) c2x = indice++;
  if (!bc2y) c2y = indice++;

  tr.valid = true;
  tr.cx1=c1x;
  tr.cy1=c1y;
  tr.cx2=c2x;
  tr.cy2=c2y;
  tr.espessura = 0;

  for (vector<trecho>::const_iterator i=delgada_doc.trechos.begin();i!=delgada_doc.trechos.end();i++)
    if (delgada_doc.TrechoExiste(*i))
      if (MesmasCotas (tr,*i)) ja_existia = true;
}
void TDo_Undo_IntroduzirTrecho::ActualDo()
{
  if (ja_existia) return;
  if (!bc1x){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = p1.x;
    delgada_doc.cotas.back().e     = eixo_x;
  };
  if (!bc1y){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = p1.y;
    delgada_doc.cotas.back().e     = eixo_y;
  };
  if (!bc2x){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = p2.x;
    delgada_doc.cotas.back().e     = eixo_x;
  };
  if (!bc2y){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = p2.y;
    delgada_doc.cotas.back().e     = eixo_y;
  };
  delgada_doc.trechos.push_back(tr);
  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_IntroduzirTrecho::ActualUndo()
{
  if (ja_existia) return;
  delgada_doc.trechos.pop_back();
  if (!bc1x) delgada_doc.cotas.pop_back();
  if (!bc1y) delgada_doc.cotas.pop_back();
  if (!bc2x) delgada_doc.cotas.pop_back();
  if (!bc2y) delgada_doc.cotas.pop_back();
  delgada_doc.DesatualizarCalculos();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_IntroduzirDistancia::TDo_Undo_IntroduzirDistancia(TDelgadaDoc& d,double c1_,double c2_,double ortogonal,eixo e_)
          : TDo_Undo (d),c1(c1_),c2(c2_),e(e_)
{
  bc1 = bc2 = false;

  dist.valid = true;
  dist.posicao_ortogonal = ortogonal;

  for (vector<cota>::const_iterator i=delgada_doc.cotas.begin();i!=delgada_doc.cotas.end();i++)
    if (delgada_doc.CotaExiste(*i)){
      int indice = i - delgada_doc.cotas.begin();
      if ((*i).e==e && (*i).valor==c1) {bc1 = true; dist.c1 = indice;};
      if ((*i).e==e && (*i).valor==c2) {bc2 = true; dist.c2 = indice;};
    };
  int indice = delgada_doc.cotas.size();
  if (!bc1) dist.c1 = indice++;
  if (!bc2) dist.c2 = indice++;
}
void TDo_Undo_IntroduzirDistancia::ActualDo()
{
  if (!bc1){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = c1;
    delgada_doc.cotas.back().e     = e;
  };
  if (!bc2){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = c2;
    delgada_doc.cotas.back().e     = e;
  };
  delgada_doc.distancias.push_back(dist);
}
void TDo_Undo_IntroduzirDistancia::ActualUndo()
{
  delgada_doc.distancias.pop_back();
  if (!bc1) delgada_doc.cotas.pop_back();
  if (!bc2) delgada_doc.cotas.pop_back();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_Apagar::TDo_Undo_Apagar (TDelgadaDoc& d, oque que_ ,int indice)
           :TDo_Undo(d),que(que_),i(indice)
{
  valid = false;
  int size = que==trecho?delgada_doc.trechos.size():
             que==distancia?delgada_doc.distancias.size():
             delgada_doc.cotas.size();
  CHECK (i>=0 && i<size);
}
void TDo_Undo_Apagar::ActualDo()
{
  if (que==trecho) swap(delgada_doc.trechos[i].valid,valid);
  if (que==distancia) swap(delgada_doc.distancias[i].valid,valid);
  if (que==cota) swap(delgada_doc.cotas[i].valid,valid);
  delgada_doc.DesatualizarCalculos();
}
void TDo_Undo_Apagar::ActualUndo()
{
  ActualDo();
}

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

TDo_Undo_Origem::TDo_Undo_Origem (TDelgadaDoc& d, template_point<double> p_)
           :TDo_Undo(d),p(p_)
{
  bcx = bcy = false;

  for (vector<cota>::const_iterator i=delgada_doc.cotas.begin();i!=delgada_doc.cotas.end();i++)
    if (delgada_doc.CotaExiste(*i)){
      int indice = i - delgada_doc.cotas.begin();
      if ((*i).e==eixo_x && (*i).valor==p.x) {bcx = true; cxOrigem = indice;};
      if ((*i).e==eixo_y && (*i).valor==p.y) {bcy = true; cyOrigem = indice;};
    };
  int indice = delgada_doc.cotas.size();
  if (!bcx) cxOrigem = indice++;
  if (!bcy) cyOrigem = indice++;
}
void TDo_Undo_Origem::ActualDo()
{
  if (!bcx){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = p.x;
    delgada_doc.cotas.back().e     = eixo_x;
  };
  if (!bcy){
    delgada_doc.cotas.push_back(cota());
    delgada_doc.cotas.back().valid = true;
    delgada_doc.cotas.back().valor = p.y;
    delgada_doc.cotas.back().e     = eixo_y;
  };
  swap (cxOrigem,delgada_doc.cxOrigem);
  swap (cyOrigem,delgada_doc.cyOrigem);
}
void TDo_Undo_Origem::ActualUndo()
{
  swap (cxOrigem,delgada_doc.cxOrigem);
  swap (cyOrigem,delgada_doc.cyOrigem);
  if (!bcx) delgada_doc.cotas.pop_back();
  if (!bcy) delgada_doc.cotas.pop_back();
}

