//
//  FLEXO II - Programa para clculo de propriedades de sees delgadas
//  Copyright (C) 1999  Maurcio C. Antunes (mantunes@sc.usp.br)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//

#if !defined(do_undo_inheriteds_hpp)
#define do_undo_inheriteds_hpp

#include <do_undo.hpp>
#include <pointsizerect.hpp>
#include <delgadadoc.h>

class TDo_Undo_Introduzircotas : public TDo_Undo{
public:
  TDo_Undo_Introduzircotas(TDelgadaDoc& d,double posicao,eixo dir);
  TDo_Undo_Introduzircotas(TDelgadaDoc& d,double posicaox,double posicaoy);
protected:
  void ActualDo();
  void ActualUndo();
  bool cota_x,cota_y;
  double pos_x,pos_y;
};

class TDo_Undo_Introduzirdistancia : public TDo_Undo{
public:
  TDo_Undo_Introduzirdistancia (TDelgadaDoc& d,int c1_,int c2_, double posicao_);
protected:
  void ActualDo();
  void ActualUndo();
  int c1,c2;
  double posicao;
};

class TDo_Undo_EscalarCotas : public TDo_Undo{
public:
  TDo_Undo_EscalarCotas(TDelgadaDoc& d,double escala_);
protected:
  void ActualDo();
  void ActualUndo();
  void Before(DoingOrUndoing);
  bool JaFoiEscalado;
  double escala;
  std::vector<cota> cotas;
};

class TDo_Undo_ModificarCotas : public TDo_Undo{
public:
  TDo_Undo_ModificarCotas(TDelgadaDoc& d,const std::vector<bool>& v,double deslocamento_);
protected:
  void ActualDo();
  void ActualUndo();
  void SomarNasCotas (const double&);
  double deslocamento;
  std::vector<bool> vbool;
};

class TDo_Undo_ModificarEspessura : public TDo_Undo{
public:
  TDo_Undo_ModificarEspessura(TDelgadaDoc& d,int id_trecho_, double espessura_);
  TDo_Undo_ModificarEspessura(TDelgadaDoc& d,double espessura_);
protected:
  void ActualDo();
  void ActualUndo();
  int id_trecho;
  double espessura;
  bool todos_os_trechos;
  std::vector<double> espessuras;
};

class TDo_Undo_FixarOrigem : public TDo_Undo {
public:
  TDo_Undo_FixarOrigem(TDelgadaDoc& d,int cx_,int cy_);
protected:
  void ActualDo();
  void ActualUndo();
  int cx,cy;
};

class TDo_Undo_ApagarCompletamente : public TDo_Undo {
public:
  enum partes {secao,distancias,tudo};
  TDo_Undo_ApagarCompletamente (TDelgadaDoc& d, partes oque);
protected:
  void ActualDo();
  void ActualUndo();
  bool delsecao,deldistancias,delcotas;
  int cxOrigem,cyOrigem;
  std::vector<trecho> t;
  std::vector<DistanciaEntreCotas> d;
  std::vector<cota> c;
};

class TDo_Undo_IntroduzirVertices : public TDo_Undo{
public:
  TDo_Undo_IntroduzirVertices(TDelgadaDoc& d,template_point<double> posicao);
protected:
  void ActualDo();
  void ActualUndo();
  bool cota_x,cota_y;
  double pos_x,pos_y;
};

class TDo_Undo_IntroduzirTrecho : public TDo_Undo {
public:
  TDo_Undo_IntroduzirTrecho(TDelgadaDoc& d,template_point<double> p1_,template_point<double> p2_);
protected:
  void ActualDo();
  void ActualUndo();
  template_point<double> p1,p2;
  int c1x,c1y,c2x,c2y;
  bool bc1x,bc1y,bc2x,bc2y;
  bool ja_existia;
  trecho tr;
};

class TDo_Undo_IntroduzirDistancia : public TDo_Undo{
public:
  TDo_Undo_IntroduzirDistancia (TDelgadaDoc& d,double c1_,double c2_, double ortogonal,eixo e_);
protected:
  void ActualDo();
  void ActualUndo();
  double c1,c2;
  eixo e;
  bool bc1,bc2;
  DistanciaEntreCotas dist;
};

class TDo_Undo_Apagar : public TDo_Undo {
public:
  enum oque {trecho,distancia,cota};
  TDo_Undo_Apagar (TDelgadaDoc& d, oque que_ ,int indice);
protected:
  void ActualDo();
  void ActualUndo();
  oque que;
  int i;
  bool valid;
};

class TDo_Undo_Origem : public TDo_Undo
{
public:
  TDo_Undo_Origem (TDelgadaDoc& d, template_point<double> p_);
protected:
  void ActualDo();
  void ActualUndo();
  template_point<double> p;
  bool bcx,bcy;
  int cxOrigem,cyOrigem;
};

#endif // do_undo_inheriteds_hpp
